@@ -102,6 +102,7 @@ my $num_msgs = -1;
 foreach $lang (@langs) {
    my $aref = $msgs{$lang};
    $num_msgs = scalar @$aref if scalar @$aref > $num_msgs;
+   unlink "$lang.todo";
 }
 
 foreach $lang (@langs) {
@@ -119,9 +120,10 @@ foreach $lang (@langs) {
    print OUT pack("CCn", $major, $minor, $num_msgs) or die $!;
 
    my $buff = '';
+   my $missing = 0;
+   my $retranslate = 0;
 
-   my $n;
-   for $n (0 .. $num_msgs - 1) {
+   for my $n (0 .. $num_msgs - 1) {
       my $warned = 0;
       my $msg = $$aref[$n];
       if (!defined $msg) {
@@ -131,7 +133,7 @@ foreach $lang (@langs) {
 	    # don't report if we have a parent (as the omission will be
 	    # reported there)
 	    if (defined $msg && $msg ne '' && !defined $parentaref) {
-	       print STDERR "Warning: message $n not in language $lang\n";
+	       $missing++;
 	       open TODO, ">>$lang.todo" or die $!;
 	       print TODO $raw{$n}, "\n";
 	       close TODO;
@@ -140,13 +142,17 @@ foreach $lang (@langs) {
 	       $msg = '' if !defined $msg;
 	    }
 	 }
+      } else {
+	 if ($lang ne 'en') {
+	     sanity_check("Message $n in language $lang", $msg, ${$msgs{'en'}}[$n]);
+	 }
       }
       if (defined($raw{$n})
-          && $raw{$n} =~ /^\s*#\s*TRANSLATE\b[- \ta-z]*\b$lang\b/m) {
+          && $raw{$n} =~ /^\s*#\s*TRANSLATE\b[- \ta-z]*\b$lang\b([^-]|$)/m) {
 	 if ($warned) {
 	    print STDERR "Warning: message $n missing and also marked for retranslation for language $lang\n";
 	 } else {
-	    print STDERR "Warning: message $n needs retranslating for language $lang\n";
+	    ++$retranslate;
 	    open TODO, ">>$lang.todo" or die $!;
 	    print TODO $raw{$n}, "\n";
 	    close TODO;
@@ -154,9 +160,22 @@ foreach $lang (@langs) {
       }
       $buff .= $msg . "\0";
    }
-   
+
    print OUT pack('N',length($buff)), $buff or die $!;
    close OUT or die $!;
+
+   if ($missing || $retranslate) {
+       print STDERR "Warning: see $lang.todo for ";
+       if ($missing) {
+	   print STDERR "$missing missing message(s)";
+	   if ($retranslate) {
+	       print STDERR " and $retranslate requiring retranslation";
+	   }
+       } else {
+	   print STDERR "$retranslate message(s) requiring retranslation";
+       }
+       print STDERR "\n";
+   }
 }
 
 my $num_dontextract = -1;
@@ -174,6 +193,7 @@ foreach $lang (@langs) {
    print OUT "#define N_DONTEXTRACTMSGS ", $num_dontextract, "\n";
    print OUT "static unsigned char dontextractmsgs[] =";
 
+   my $missing = 0;
    for my $n (0 .. $num_dontextract - 1) {
       print OUT "\n";
 
@@ -190,7 +210,7 @@ foreach $lang (@langs) {
 	    $msg = ${$dontextract{'en'}}[$n];
 	    # don't report if we have a parent (as the omission will be reported there)
 	    if (defined $msg && $msg ne '' && !defined $parentaref) {
-	       print STDERR "Warning: message ", $dontextract_threshold + $n, " not in language $lang\n";
+	       ++$missing;
 	       open TODO, ">>$lang.todo" or die $!;
 	       print TODO $raw{$dontextract_threshold + $n}, "\n";
 	       close TODO;
@@ -198,6 +218,10 @@ foreach $lang (@langs) {
 	       $msg = '';
 	    }
 	 }
+      } else {
+	 if ($lang ne 'en') {
+	     sanity_check("Message $n in language $lang", $msg, $$aref[$n]);
+	 }
       }
       $msg =~ s/\\/\\\\/g;
       $msg =~ s/"/\\"/g;
@@ -212,6 +236,10 @@ foreach $lang (@langs) {
    }
    print OUT ";\n";
    close OUT or die $!;
+
+   if ($missing) {
+       print STDERR "Warning: see $lang.todo for $missing missing \"don't extract\" message(s)\n";
+   }
 }
 
 #for (sort {$a<=>$b} keys %raw) {print "$_ = [$raw{$_}]\n";}
@@ -250,3 +278,49 @@ sub char_to_utf8 {
    return $result;
 }
 
+sub sanity_check {
+   my ($where, $msg, $orig) = @_;
+   # check printf-like specifiers match
+   # allow valid printf specifiers, or %<any letter> to support strftime
+   # and other printf-like formats.
+   my @pcent_m = grep /\%/, split /(%(?:[-#0 +'I]*(?:[0-9]*|\*|\*m\$)(?:\.[0-9]*)?(?:hh|ll|[hlLqjzt])?[diouxXeEfFgGaAcsCSpn]|[a-zA-Z]))/, $msg;
+   my @pcent_o = grep /\%/, split /(%(?:[-#0 +'I]*(?:[0-9]*|\*|\*m\$)(?:\.[0-9]*)?(?:hh|ll|[hlLqjzt])?[diouxXeEfFgGaAcsCSpn]|[a-zA-Z]))/, $orig;
+   while (scalar @pcent_m || scalar @pcent_o) {
+       if (!scalar @pcent_m) {
+	   print STDERR "Warning: $where misses out \%spec $pcent_o[0]\n";
+       } elsif (!scalar @pcent_o) {
+	   print STDERR "Warning: $where has extra \%spec $pcent_m[0]\n";
+       } elsif ($pcent_m[0] ne $pcent_o[0]) {
+	   print STDERR "Warning: $where has \%spec $pcent_m[0] instead of $pcent_o[0]\n";
+       }
+       pop @pcent_m;
+       pop @pcent_o;
+   }
+
+   # Check for missing (or added) elipses (...)
+   if ($msg =~ /\.\.\./ && $orig !~ /\.\.\./) {
+       print STDERR "Warning: $where has elipses but original doesn't\n";
+   } elsif ($msg !~ /\.\.\./ && $orig =~ /\.\.\./) {
+       print STDERR "Warning: $where is missing elipses\n";
+   }
+
+   # Check for missing (or added) menu shortcut (@)
+   if ($msg =~ /\@[A-Za-z]/ && $orig !~ /\@[A-Za-z]/) {
+       print STDERR "Warning: $where has menu shortcut but original doesn't\n";
+   } elsif ($msg !~ /\@[A-Za-z]/ && $orig =~ /\@[A-Za-z]/) {
+       print STDERR "Warning: $where is missing menu shortcut\n";
+   }
+
+   # Check for missing (or added) menu accelerator "##"
+   if ($msg =~ /\#\#/ && $orig !~ /\#\#/) {
+       print STDERR "Warning: $where has menu accelerator but original doesn't\n";
+   } elsif ($msg !~ /\#\#/ && $orig =~ /\#\#/) {
+       print STDERR "Warning: $where is missing menu accelerator\n";
+   } elsif ($orig =~ /\#\#(.*)/) {
+       my $acc_o = $1;
+       my ($acc_m) = $msg =~ /\#\#(.*)/;
+       if ($acc_o ne $acc_m) {
+	   print STDERR "Warning: $where has menu accelerator $acc_m instead of $acc_o\n";
+       }
+   }
+}
