@@ -4,7 +4,7 @@
 //  About box handling for Aven.
 //
 //  Copyright (C) 2001-2003 Mark R. Shinwell.
-//  Copyright (C) 2001,2002,2003,2004 Olly Betts
+//  Copyright (C) 2001,2002,2003,2004,2005 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -36,16 +36,15 @@
 BEGIN_EVENT_TABLE(AboutDlg, wxDialog)
 END_EVENT_TABLE()
 
-AboutDlg::AboutDlg(wxWindow* parent) :
+AboutDlg::AboutDlg(wxWindow* parent, const wxString & icon_path) :
     wxDialog(parent, 500, wxString::Format(msg(/*About %s*/205), APP_NAME))
 {
     wxBoxSizer* horiz = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer* vert = new wxBoxSizer(wxVERTICAL);
 
     if (!bitmap.Ok()) {
-	bitmap.LoadFile(wxString(msg_cfgpth()) + wxCONFIG_PATH_SEPARATOR +
-			wxString("icons") + wxCONFIG_PATH_SEPARATOR +
-			wxString("aven-about.png"), wxBITMAP_TYPE_PNG);
+	bitmap.LoadFile(icon_path + "aven-about.png", wxBITMAP_TYPE_PNG);
+	bitmap_icon.LoadFile(icon_path + "aven.png", wxBITMAP_TYPE_PNG);
     }
     if (bitmap.Ok()) {
 	wxStaticBitmap* static_bitmap = new wxStaticBitmap(this, 501, bitmap);
@@ -56,10 +55,23 @@ AboutDlg::AboutDlg(wxWindow* parent) :
 
     wxString id = wxString(APP_NAME" "VERSION"\n");
     id += msg(/*Survey visualisation tool*/209);
-    wxStaticText* title = new wxStaticText(this, 502, id);
-    wxStaticText* copyright = new wxStaticText(this, 503,
-	    wxString::Format(AVEN_COPYRIGHT_MSG"\n"COPYRIGHT_MSG,
-			     msg(/*&copy;*/0), msg(/*&copy;*/0)));
+    wxBoxSizer* title = new wxBoxSizer(wxHORIZONTAL);
+    if (bitmap_icon.Ok()) {
+	title->Add(new wxStaticBitmap(this, 599, bitmap_icon), 0, wxALIGN_CENTRE_VERTICAL|wxRIGHT, 8);
+    }
+    title->Add(new wxStaticText(this, 502, id), 0, wxALL, 2);
+
+    wxString copyright_msg = COPYRIGHT_MSG"\n"AVEN_COPYRIGHT_MSG;
+    const char * csign = msg(/*&copy;*/0);
+    if (strcmp(csign, "(C)") != 0) {
+	size_t csign_len = strlen(csign);
+	size_t i = 0;
+	while ((i = copyright_msg.find("(C)", i)) != wxString::npos) {
+	    copyright_msg.replace(i, 3, csign, csign_len);
+	    i += csign_len;
+	}
+    }
+    wxStaticText* copyright = new wxStaticText(this, 503, copyright_msg);
 
     wxString licence_str;
     wxString l(msg(/*This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public Licence as published by the Free Software Foundation; either version 2 of the Licence, or (at your option) any later version.*/219));
@@ -100,9 +112,10 @@ AboutDlg::AboutDlg(wxWindow* parent) :
     vert->Add(new wxStaticText(this, 505, msg(/*System Information:*/390)),
 	      0, wxLEFT | wxRIGHT, 20);
 
-#ifdef unix
-    // On Unix, wx reports the OS that we were *built* on, which may have
-    // be a different OS or kernel version to what we're running on.
+#if defined __UNIX__ && !wxCHECK_VERSION(2,5,4)
+    // On Unix, older wx versions report the OS that we were *built* on, which
+    // may be a different OS or kernel version to what we're running on.
+    // I submitted a patch to fix this which was applied in 2.5.4.
     wxString info;
     {
 	char buf[80];
@@ -122,6 +135,25 @@ AboutDlg::AboutDlg(wxWindow* parent) :
 #endif
     info += '\n';
     info += wxVERSION_STRING;
+#ifdef __WXGTK__
+#if defined __WXGTK24__
+    info += " (GTK+ >= 2.4)";
+#elif defined __WXGTK20__
+    info += " (GTK+ >= 2.0)";
+#elif defined __WXGTK12__
+    info += " (GTK+ >= 1.2)";
+#else
+    info += " (GTK+ < 1.2)";
+#endif
+#elif defined __WXMOTIF__
+#if defined __WXMOTIF20__
+    info += " (Motif >= 2.0)";
+#else
+    info += " (Motif < 2.0)";
+#endif
+#elif defined __WXX11__
+    info += " (X11)";
+#endif
     info += '\n';
     int bpp = wxDisplayDepth();
     info += wxString::Format("Display Depth: %d bpp", bpp);
