@@ -43,6 +43,12 @@
 #include "avenprcore.h"
 #include "debug.h"
 
+#if defined __WXMSW__ || defined __WXMAC__
+# include <wx/dcprint.h>
+#else
+# include <wx/dcps.h>
+#endif
+
 layout::layout(wxPageSetupDialogData* data)
 	: Labels(false), Crosses(false), Shots(true), Surface(false),
 	  SkipBlank(false), Border(true), Cutlines(true), Raw(false),
@@ -50,13 +56,21 @@ layout::layout(wxPageSetupDialogData* data)
 	  view(PLAN), scX(1), scY(1), xMin(0), xMax(-1), yMin(0), yMax(-1),
 	  pagesX(1), pagesY(1), pages(1), xOrg(0), yOrg(0), footer(NULL)
 {
-    // Get paper size information.
-    PaperWidth = data->GetPaperSize().GetWidth() - 
-	data->GetMarginBottomRight().x - data->GetMarginTopLeft().x;
-    PaperDepth = data->GetPaperSize().GetHeight() - 
-	data->GetMarginBottomRight().y - data->GetMarginTopLeft().y;
+    // Create a temporary wxPrinterDC/wxPostScriptDC so we can get access to
+    // the size of the printable area in mm to allow us to calculate how many
+    // pages will be needed.
+#if defined __WXMSW__ || defined __WXMAC__
+    wxPrinterDC pdc(data->GetPrintData());
+#else
+    wxPostScriptDC pdc(data->GetPrintData());
+#endif
+    int width, depth;
+    pdc.GetSizeMM(&width, &depth);
+    width -= data->GetMarginBottomRight().x + data->GetMarginTopLeft().x;
+    PaperWidth = width;
+    depth -= data->GetMarginBottomRight().y + data->GetMarginTopLeft().y;
     // Allow for the 10mm footer.
-    PaperDepth -= 10;
+    PaperDepth = depth - 10;
 }
 
 void
