@@ -4,7 +4,8 @@
 //  Tree control used for the survey tree.
 //
 //  Copyright (C) 2001, Mark R. Shinwell.
-//  Copyright (C) 2001-2003, Olly Betts
+//  Copyright (C) 2001-2003,2005 Olly Betts
+//  Copyright (C) 2005 Martin Green
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -30,15 +31,16 @@
 
 BEGIN_EVENT_TABLE(AvenTreeCtrl, wxTreeCtrl)
     EVT_MOTION(AvenTreeCtrl::OnMouseMove)
+    EVT_LEAVE_WINDOW(AvenTreeCtrl::OnLeaveWindow)
     EVT_TREE_SEL_CHANGED(-1, AvenTreeCtrl::OnSelChanged)
     EVT_CHAR(AvenTreeCtrl::OnKeyPress)
 END_EVENT_TABLE()
 
 AvenTreeCtrl::AvenTreeCtrl(MainFrm* parent, wxWindow* window_parent) :
-    wxTreeCtrl(window_parent, -1, wxDefaultPosition),
+    wxTreeCtrl(window_parent),
     m_Parent(parent),
     m_Enabled(false),
-    m_LastItem(wxTreeItemId(-1)),
+    m_LastItem(),
     m_BackgroundColour(GetBackgroundColour()),
     m_SelValid(false)
 {
@@ -51,21 +53,33 @@ void AvenTreeCtrl::OnMouseMove(wxMouseEvent& event)
     if (m_Enabled) {
 	int flags;
 	wxTreeItemId pos = HitTest(event.GetPosition(), flags);
+	if (!(flags & TREE_MASK)) {
+	    pos = wxTreeItemId();
+	}
 	if (pos != m_LastItem) {
-	    if (m_LastItem != wxTreeItemId(-1)) {
+	    if (m_LastItem.IsOk()) {
 		SetItemBackgroundColour(m_LastItem, m_BackgroundColour);
 	    }
-	    if (flags & TREE_MASK) {
+	    if (pos.IsOk()) {
 		SetItemBackgroundColour(pos, wxColour(180, 180, 180));
 		m_Parent->DisplayTreeInfo(GetItemData(pos));
-		m_LastItem = pos;
 	    } else {
 		m_Parent->DisplayTreeInfo(NULL);
 	    }
+	    m_LastItem = pos;
 	}
     }
 }
 
+void AvenTreeCtrl::OnLeaveWindow(wxMouseEvent&)
+{
+    if (m_LastItem.IsOk()) {
+	SetItemBackgroundColour(m_LastItem, m_BackgroundColour);
+	m_LastItem = wxTreeItemId();
+    }
+    m_Parent->DisplayTreeInfo(NULL);
+}
+
 void AvenTreeCtrl::SetEnabled(bool enabled)
 {
     m_Enabled = enabled;
@@ -83,6 +97,7 @@ void AvenTreeCtrl::OnSelChanged(wxTreeEvent&)
 bool AvenTreeCtrl::GetSelectionData(wxTreeItemData** data)
 {
     assert(m_Enabled);
+    assert(data);
 
     if (!m_SelValid) {
 	return false;
@@ -104,16 +119,45 @@ void AvenTreeCtrl::UnselectAll()
 
 void AvenTreeCtrl::DeleteAllItems()
 {
-    m_LastItem = -1;
+    m_Enabled = false;
+    m_LastItem = wxTreeItemId();
     m_SelValid = false;
-    wxTreeCtrl::DeleteAllItems();    
+    wxTreeCtrl::DeleteAllItems();
 }
 
 void AvenTreeCtrl::OnKeyPress(wxKeyEvent &e)
 {
-    if (e.m_keyCode == WXK_ESCAPE) {
-	m_Parent->ClearTreeSelection();
-    } else {
-	e.Skip();
+    switch (e.m_keyCode) {
+	case WXK_ESCAPE:
+	    m_Parent->ClearTreeSelection();
+	    break;
+	case WXK_RETURN: {
+	    wxTreeItemId id = GetSelection();
+	    if (id.IsOk()) {
+		if (ItemHasChildren(id)) {
+		    // If on a branch, expand/contract it.
+		    if (IsExpanded(id)) {
+			Collapse(id);
+		    } else {
+			Expand(id);
+		    }
+		} else {
+		    // FIXME if on a station, show information on that station
+		    // or something?
+		}
+	    }
+	    break;
+	}
+	case WXK_LEFT: case WXK_RIGHT: case WXK_UP: case WXK_DOWN:
+	case WXK_HOME: case WXK_END: case WXK_PAGEUP: case WXK_PAGEDOWN:
+	// PRIOR/NEXT seem to actually be PAGEUP/PAGEDOWN (tested on wxGtk)
+	case WXK_PRIOR: case WXK_NEXT:
+	    e.Skip();
+	    break;
+	default:
+	    // Pass key event to MainFrm which will pass to GfxCore which will
+	    // pass to GUIControl.
+	    m_Parent->OnKeyPress(e);
+	    break;
     }
 }
