@@ -1,6 +1,6 @@
 /* commands.h
  * Header file for code for directives
- * Copyright (C) 1994-2001 Olly Betts
+ * Copyright (C) 1994-2002 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +20,15 @@
 void handle_command(void);
 void default_all(settings *s);
 
+void default_units(settings *s);
+void default_calib(settings *s);
+
 void free_settings(settings *p);
 
+extern char *buffer;
 void get_token(void);
 
 typedef struct { const char *sz; int tok; } sztok;
 int match_tok(const sztok *tab, int tab_size);
 
 #define TABSIZE(T) ((sizeof(T) / sizeof(sztok)) - 1)
-
-extern reading default_order[];
