@@ -1,6 +1,7 @@
 /* datain.c
  * Reads in survey files, dealing with special characters, keywords & data
- * Copyright (C) 1991-2003 Olly Betts
+ * Copyright (C) 1991-2003,2005 Olly Betts
+ * Copyright (C) 2004 Simeon Warner
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +42,10 @@
 
 #define var(I) (pcs->Var[(I)])
 
+/* true if x is not-a-number value in Compass (999.0 or -999.0)    */
+/* Compass uses 999.0 but understands Karst data which used -999.0 */
+#define is_compass_NaN(x) ( fabs(fabs(x)-999.0) <  EPSILON )
+
 int ch;
 
 typedef enum {
@@ -58,6 +63,12 @@ static real value[Fr - 1];
 static real variance[Fr - 1];
 #define VAR(N) variance[(N)-1]
 
+/* style functions */
+static int data_normal(void);
+static int data_cartesian(void);
+static int data_nosurvey(void);
+static int data_ignore(void);
+
 void
 get_pos(filepos *fp)
 {
@@ -273,7 +284,7 @@ read_reading(reading r, bool f_optional)
       case Dz: q = Q_DZ; break;
       case FrCount: case ToCount: q = Q_COUNT; break;
       default:
-	q = Q_NULL; /* Suppress compiler warning */; 
+	q = Q_NULL; /* Suppress compiler warning */;
 	BUG("Unexpected case");
    }
    VAR(r) = var(q);
@@ -290,7 +301,7 @@ read_bearing_or_omit(reading r)
       case Comp: q = Q_BEARING; break;
       case BackComp: q = Q_BACKBEARING; break;
       default:
-	q = Q_NULL; /* Suppress compiler warning */; 
+	q = Q_NULL; /* Suppress compiler warning */;
 	BUG("Unexpected case");
    }
    VAR(r) = var(q);
@@ -418,7 +429,7 @@ data_file(const char *pth, const char *fnm)
       pcs = pcsNew;
 
       t = ((short*)osmalloc(ossizeof(short) * 257)) + 1;
-      
+
       t[EOF] = SPECIAL_EOL;
       memset(t, 0, sizeof(short) * 33);
       for (i = 33; i < 127; i++) t[i] = SPECIAL_NAMES;
@@ -448,12 +459,14 @@ data_file(const char *pth, const char *fnm)
    if (fmt == FMT_DAT) {
       while (!feof(file.fh) && !ferror(file.fh)) {
 	 static reading compass_order[] = {
-	    Fr, To, Tape, Comp, Clino, Ignore, Ignore, Ignore, Ignore,
+	    Fr, To, Tape, CompassDATComp, CompassDATClino,
+	    Ignore, Ignore, Ignore, Ignore,
 	    CompassDATFlags, IgnoreAll
 	 };
 	 static reading compass_order_backsights[] = {
-	    Fr, To, Tape, Comp, Clino, Ignore, Ignore, Ignore, Ignore,
-	    BackComp, BackClino,
+	    Fr, To, Tape, CompassDATComp, CompassDATClino,
+	    Ignore, Ignore, Ignore, Ignore,
+	    CompassDATBackComp, CompassDATBackClino,
 	    CompassDATFlags, IgnoreAll
 	 };
 	 /* <Cave name> */
@@ -471,6 +484,7 @@ data_file(const char *pth, const char *fnm)
 	 process_eol();
 	 /* SURVEY DATE: 7 10 79  COMMENT:<Long name> */
 	 /* Note: Larry says a 2 digit year is always 19XX */
+	 /* NB order is *month* *day* year */
 	 get_token();
 	 get_token();
 	 /* if (ch != ':') ... */
@@ -543,7 +557,7 @@ data_file(const char *pth, const char *fnm)
 	 pcs->ordering = NULL;
 	 free_settings(pcs);
 	 pcs = pcsParent;
-      }					  
+      }
    } else if (fmt == FMT_MAK) {
       nextch_handling_eol();
       while (!feof(file.fh) && !ferror(file.fh)) {
@@ -629,7 +643,7 @@ data_file(const char *pth, const char *fnm)
 	 SVX_ASSERT(pcsParent);
 	 free_settings(pcs);
 	 pcs = pcsParent;
-      }					  
+      }
    } else {
       while (!feof(file.fh) && !ferror(file.fh)) {
 	 if (!process_non_data_line()) {
@@ -1233,7 +1247,7 @@ process_cartesian(prefix *fr, prefix *to, bool fToFirst)
    return 1;
 }
 
-extern int
+static int
 data_cartesian(void)
 {
    prefix *fr = NULL, *to = NULL;
@@ -1391,7 +1405,7 @@ process_cylpolar(prefix *fr, prefix *to, bool fToFirst, bool fDepthChange)
 
 /* Process tape/compass/clino, diving, and cylpolar styles of survey data
  * Also handles topofil (fromcount/tocount or count) in place of tape */
-extern int
+static int
 data_normal(void)
 {
    prefix *fr = NULL, *to = NULL;
@@ -1406,7 +1420,7 @@ data_normal(void)
    reading *ordering;
 
    VAL(Tape) = 0;
-   VAL(Comp) = VAL(BackComp) = HUGE_VAL;
+   VAL(Comp) = VAL(BackComp) = HUGE_REAL;
    VAL(FrCount) = VAL(ToCount) = 0;
    VAL(FrDepth) = VAL(ToDepth) = 0;
 
@@ -1455,7 +1469,7 @@ data_normal(void)
 	     break;
 	   default:
 	     compile_error_skip(/*Found `%s', expecting `F' or `B'*/131,
-			        buffer);
+				buffer);
 	     process_eol();
 	     return 0;
 	  }
@@ -1478,11 +1492,8 @@ data_normal(void)
 	  read_reading(ToCount, fFalse);
 	  fTopofil = fTrue;
 	  break;
-       case Comp:
-	  read_bearing_or_omit(Comp);
-	  break;
-       case BackComp:
-	  read_bearing_or_omit(BackComp);
+       case Comp: case BackComp:
+	  read_bearing_or_omit(*ordering);
 	  break;
        case Clino:
 	  read_reading(Clino, fTrue);
@@ -1506,11 +1517,8 @@ data_normal(void)
 	  }
 	  backctype = CTYPE_READING;
 	  break;
-       case FrDepth:
-	  read_reading(FrDepth, fFalse);
-	  break;
-       case ToDepth:
-	  read_reading(ToDepth, fFalse);
+       case FrDepth: case ToDepth:
+	  read_reading(*ordering, fFalse);
 	  break;
        case Depth:
 	  VAL(FrDepth) = VAL(ToDepth);
@@ -1521,6 +1529,32 @@ data_normal(void)
 	  VAL(FrDepth) = 0;
 	  read_reading(ToDepth, fFalse);
 	  break;
+       case CompassDATComp:
+	  read_bearing_or_omit(Comp);
+	  if (is_compass_NaN(VAL(Comp))) VAL(Comp) = HUGE_REAL;
+	  break;
+       case CompassDATBackComp:
+	  read_bearing_or_omit(BackComp);
+	  if (is_compass_NaN(VAL(BackComp))) VAL(BackComp) = HUGE_REAL;
+	  break;
+       case CompassDATClino:
+	  read_reading(Clino, fFalse);
+	  if (is_compass_NaN(VAL(Clino))) {
+	     VAL(Clino) = HUGE_REAL;
+	     ctype = CTYPE_OMIT;
+	  } else {
+	     ctype = CTYPE_READING;
+	  }
+	  break;
+       case CompassDATBackClino:
+	  read_reading(BackClino, fFalse);
+	  if (is_compass_NaN(VAL(BackClino))) {
+	     VAL(BackClino) = HUGE_REAL;
+	     backctype = CTYPE_OMIT;
+	  } else {
+	     backctype = CTYPE_READING;
+	  }
+	  break;
        case CompassDATFlags:
 	  if (ch == '#') {
 	     nextch();
@@ -1530,6 +1564,12 @@ data_normal(void)
 		nextch();
 		while (ch >= 'A' && ch <= 'Z') {
 		   compass_dat_flags |= BIT(ch - 'A');
+		   /* FIXME: we currently understand X (exclude data)
+		    * but should also handle at least some of:
+		    * L exclude from length
+		    * P no plot
+		    * C no adjustment
+		    */
 		   nextch();
 		}
 		if (ch == '#') {
@@ -1559,15 +1599,15 @@ data_normal(void)
 	     /* Note: frdepth == todepth test works regardless of fDepthChange
 	      * (frdepth always zero, todepth is change of depth) and also
 	      * works for STYLE_NORMAL (both remain 0) */
- 	     if (TSTBIT(pcs->infer, INFER_EQUATES) &&
+	     if (TSTBIT(pcs->infer, INFER_EQUATES) &&
 		 VAL(Tape) == (real)0.0 && VAL(FrDepth) == VAL(ToDepth)) {
 		process_equate(fr, to);
 		goto inferred_equate;
 	     }
 	     if (fRev) {
 		prefix *t = fr;
-	   	fr = to;
-	    	to = t;
+		fr = to;
+		to = t;
 	     }
 	     if (fTopofil) {
 		VAL(Tape) *= pcs->units[Q_COUNT] * pcs->sc[Q_COUNT];
@@ -1594,7 +1634,7 @@ data_normal(void)
 		BUG("bad style");
 	     }
 	     if (!r) skipline();
-	     
+
 	     /* Swap fr and to back to how they were for next line */
 	     if (fRev) {
 		prefix *t = fr;
@@ -1611,7 +1651,7 @@ data_normal(void)
 	  /* this is also used if clino reading is the omit character */
 	  VAL(Clino) = VAL(BackClino) = 0;
 
-          inferred_equate:
+	  inferred_equate:
 
 	  fMulti = fTrue;
 	  while (1) {
@@ -1644,7 +1684,7 @@ data_normal(void)
 	     /* Note: frdepth == todepth test works regardless of fDepthChange
 	      * (frdepth always zero, todepth is change of depth) and also
 	      * works for STYLE_NORMAL (both remain 0) */
- 	     if (TSTBIT(pcs->infer, INFER_EQUATES) &&
+	     if (TSTBIT(pcs->infer, INFER_EQUATES) &&
 		 VAL(Tape) == (real)0.0 && VAL(FrDepth) == VAL(ToDepth)) {
 		process_equate(fr, to);
 		process_eol();
@@ -1671,9 +1711,10 @@ data_normal(void)
 				     fDepthChange);
 		break;
 	      default:
-		r = 0; /* Suppress compiler warning */; 
+		r = 0; /* Suppress compiler warning */;
 		BUG("bad style");
 	     }
+
 	     process_eol();
 	     return r;
 	  }
@@ -1735,7 +1776,7 @@ process_nosurvey(prefix *fr, prefix *to, bool fToFirst)
    return 1;
 }
 
-extern int
+static int
 data_nosurvey(void)
 {
    prefix *fr = NULL, *to = NULL;
@@ -1817,7 +1858,7 @@ data_nosurvey(void)
 }
 
 /* totally ignore a line of survey data */
-extern int
+static int
 data_ignore(void)
 {
    skipline();
