@@ -1,7 +1,7 @@
 /* extend.c
  * Produce an extended elevation
- * Copyright (C) 1995-2002 Olly Betts
- * Copyright (C) 2004 John Pybus
+ * Copyright (C) 1995-2002,2005 Olly Betts
+ * Copyright (C) 2004,2005 John Pybus
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,37 +49,38 @@ typedef struct POINT {
    img_point p;
    const stn *stns;
    unsigned int order;
-   int fDir;
-   int fDone;
+   char dir;
+   char fDone;
+   char fBroken;
    struct POINT *next;
 } point;
 
 typedef struct LEG {
    point *fr, *to;
    const char *prefix;
-   int fDir;
-   int fDone;
+   char dir;
+   char fDone;
+   char broken;
    int flags;
    struct LEG *next;
 } leg;
 
-#define DONE        0x01
-#define BREAK_FR    0x04
-#define BREAK_TO    0x08
-#define BREAK       (BREAK_FR | BREAK_TO)
+/* Values for leg.broken: */
+#define BREAK_FR    0x01
+#define BREAK_TO    0x02
 
+/* Values for point.dir and leg.dir: */
+#define ELEFT  0x01
+#define ERIGHT 0x02
+#define ESWAP  0x04
 
-#define ELEFT  0x04
-#define ERIGHT 0x08
-#define ESWAP  0x10
+static point headpoint = {{0, 0, 0}, NULL, 0, 0, 0, 0, NULL};
 
-static point headpoint = {{0, 0, 0}, NULL, 0, 0, 0, NULL};
+static leg headleg = {NULL, NULL, NULL, 0, 0, 0, 0, NULL};
 
-static leg headleg = {NULL, NULL, NULL, 0, 0, 0, NULL};
+static img *pimg_out;
 
-static img *pimg;
-
-static void do_stn(point *, double, const char *, int);
+static void do_stn(point *, double, const char *, int, int);
 
 typedef struct pfx {
    const char *label;
@@ -125,8 +126,9 @@ find_point(const img_point *pt)
    p->p = *pt;
    p->stns = NULL;
    p->order = 0;
-   p->fDir = 0;
+   p->dir = 0;
    p->fDone = 0;
+   p->fBroken = 0;
    p->next = headpoint.next;
    headpoint.next = p;
    return p;
@@ -146,8 +148,9 @@ add_leg(point *fr, point *to, const char *prefix, int flags)
    else
       l->prefix = NULL;
    l->next = headleg.next;
-   l->fDir = 0;
+   l->dir = 0;
    l->fDone = 0;
+   l->broken = 0;
    l->flags = flags;
    headleg.next = l;
 }
@@ -220,13 +223,13 @@ delimword(char *ln, char** lr)
    } else {
       *lr = le + 1;
    }
-   
+
    *le = '\0';
    return ln;
 }
 
 static void
-parseconfigline(char *ln)
+parseconfigline(const char *fnm, char *ln)
 {
    point *p;
    const stn *s;
@@ -240,34 +243,36 @@ parseconfigline(char *ln)
 
    if (strcmp(ln, "*start")==0) {
       ln = delimword(lc, &lc);
-      if (*ln == 0) fatalerror(/*Command without station name in config, line %i*/602, lineno);
+      if (*ln == 0)
+	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       for (p = headpoint.next; p != NULL; p = p->next) {
 	 for (s = p->stns; s; s = s->next) {
 	    if (strcmp(s->label, ln)==0) {
 	       start = p;
-	       printf(msg(/*Starting from station %s*/604),ln);
+	       printf(msg(/*Starting from station %s*/512),ln);
 	       putnl();
 	       goto loopend;
 	    }
 	 }
       }
-      warning(/*Failed to find station %s in config, line %i*/603, ln, lineno);
+      warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ln);
    } else if (strcmp(ln, "*eleft")==0) {
       char *ll = delimword(lc, &lc);
-      if (*ll == 0) fatalerror(/*Command without station name in config, line %i*/602,lineno);
+      if (*ll == 0)
+	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       ln = delimword(lc, &lc);
       if (*ln == 0) { /* One argument, look for point to switch at. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    for (s = p->stns; s; s = s->next) {
 	       if (strcmp(s->label, ll)==0) {
-		  printf(msg(/*Plotting to the left from station %s*/605),ll);
+		  printf(msg(/*Extending to the left from station %s*/513), ll);
 		  putnl();
-		  p->fDir = ELEFT;
+		  p->dir = ELEFT;
 		  goto loopend;
 	       }
 	    }
 	 }
-	 warning(/*Failed to find station %s in config, line %i*/603, ll, lineno);       
+	 warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ll);
       } else { /* Two arguments look for a specific leg */
 	 for (l = headleg.next; l; l=l->next) {
 	    point * fr = l->fr;
@@ -279,9 +284,9 @@ parseconfigline(char *ln)
 		     char * lr = (b ? ll : ln);
 		     for (t=to->stns; t; t=t->next) {
 			if (strcmp(t->label,lr)==0) {
-			   printf(msg(/*Plotting to the left from leg %s -> %s*/607), s->label, t->label);
+			   printf(msg(/*Extending to the left from leg %s -> %s*/515), s->label, t->label);
 			   putnl();
-			   l->fDir=ELEFT;
+			   l->dir = ELEFT;
 			   goto loopend;
 			}
 		     }
@@ -289,24 +294,25 @@ parseconfigline(char *ln)
 	       }
 	    }
 	 }
-	 warning(/*Failed to find leg %s-%s in config, line %i*/606, ll, ln, lineno);
+	 warning_in_file(fnm, lineno, /*Failed to find leg %s -> %s*/511, ll, ln);
       }
    } else if (strcmp(ln, "*eright")==0) {
       char *ll = delimword(lc, &lc);
-      if (*ll == 0) fatalerror(/*Command without station name in config, line %i*/602,lineno);
+      if (*ll == 0)
+	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       ln = delimword(lc, &lc);
       if (*ln == 0) { /* One argument, look for point to switch at. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    for (s = p->stns; s; s = s->next) {
 	       if (strcmp(s->label, ll)==0) {
-		  printf(msg(/*Plotting to the right from station %s*/608),ll);
+		  printf(msg(/*Extending to the right from station %s*/514), ll);
 		  putnl();
-		  p->fDir = ERIGHT;
+		  p->dir = ERIGHT;
 		  goto loopend;
 	       }
 	    }
 	 }
-	 warning(/*Failed to find station %s in config, line %i*/603, ll, lineno);      
+	 warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ll);
       } else { /* Two arguments look for a specific leg */
 	 for (l = headleg.next; l; l=l->next) {
 	    point * fr = l->fr;
@@ -318,9 +324,9 @@ parseconfigline(char *ln)
 		     char * lr = (b ? ll : ln);
 		     for (t=to->stns; t; t=t->next) {
 			if (strcmp(t->label,lr)==0) {
-			   printf(msg(/*Plotting to the right from leg %s -> %s*/609), s->label, t->label);
-			   printf("\n");
-			   l->fDir=ERIGHT;
+			   printf(msg(/*Extending to the right from leg %s -> %s*/516), s->label, t->label);
+			   putnl();
+			   l->dir=ERIGHT;
 			   goto loopend;
 			}
 		     }
@@ -328,24 +334,25 @@ parseconfigline(char *ln)
 	       }
 	    }
 	 }
-	 warning(/*Failed to find leg %s-%s in config, line %i*/606, ll, ln, lineno);
+	 warning_in_file(fnm, lineno, /*Failed to find leg %s -> %s*/511, ll, ln);
       }
    } else if (strcmp(ln, "*eswap")==0) {
       char *ll = delimword(lc, &lc);
-      if (*ll == 0) fatalerror(/*Command without station name in config, line %i*/602,lineno);
+      if (*ll == 0)
+	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       ln = delimword(lc, &lc);
       if (*ln == 0) { /* One argument, look for point to switch at. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    for (s = p->stns; s; s = s->next) {
 	       if (strcmp(s->label, ll)==0) {
-		  printf(msg(/*Swapping plot direction from station %s*/615),ll);
+		  printf(msg(/*Swapping extend direction from station %s*/519),ll);
 		  putnl();
-		  p->fDir = ESWAP;
+		  p->dir = ESWAP;
 		  goto loopend;
 	       }
 	    }
 	 }
-	 warning(/*Failed to find station %s in config, line %i*/603, ll, lineno);      
+	 warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ll);
       } else { /* Two arguments look for a specific leg */
 	 for (l = headleg.next; l; l=l->next) {
 	    point * fr = l->fr;
@@ -357,9 +364,9 @@ parseconfigline(char *ln)
 		     char * lr = (b ? ll : ln);
 		     for (t=to->stns; t; t=t->next) {
 			if (strcmp(t->label,lr)==0) {
-			   printf(msg(/*Swapping plot direction from leg %s -> %s*/616), s->label, t->label);
-			   printf("\n");
-			   l->fDir = ESWAP;
+			   printf(msg(/*Swapping extend direction from leg %s -> %s*/520), s->label, t->label);
+			   putnl();
+			   l->dir = ESWAP;
 			   goto loopend;
 			}
 		     }
@@ -367,38 +374,39 @@ parseconfigline(char *ln)
 	       }
 	    }
 	 }
-	 warning(/*Failed to find leg %s-%s in config, line %i*/606, ll, ln, lineno);
+	 warning_in_file(fnm, lineno, /*Failed to find leg %s -> %s*/511, ll, ln);
       }
    } else if (strcmp(ln, "*break")==0) {
       char *ll = delimword(lc, &lc);
-      if (*ll == 0) fatalerror(/*Command without station name in config, line %i*/602,lineno);
+      if (*ll == 0)
+	 fatalerror_in_file(fnm, lineno, /*Expecting station name*/28);
       ln = delimword(lc, &lc);
       if (*ln == 0) { /* One argument, look for point to break at. */
 	 for (p = headpoint.next; p != NULL; p = p->next) {
 	    for (s = p->stns; s; s = s->next) {
 	       if (strcmp(s->label, ll)==0) {
-		  printf(msg(/*Breaking survey at station %s*/610), ll);
+		  printf(msg(/*Breaking survey loop at station %s*/517), ll);
 		  putnl();
-		  p->fDone = BREAK;
+		  p->fBroken = 1;
 		  goto loopend;
 	       }
 	    }
 	 }
-	 warning(/*Failed to find station %s in config, line %i*/603, ll, lineno);  
+	 warning_in_file(fnm, lineno, /*Failed to find station %s*/510, ll);
       } else { /* Two arguments look for a specific leg */
 	 for (l = headleg.next; l; l=l->next) {
 	    point * fr = l->fr;
 	    point * to = l->to;
-	    if (fr && to ) {
+	    if (fr && to) {
 	       for (s=fr->stns; s; s=s->next) {
 		  int b = 0;
 		  if (strcmp(s->label,ll)==0 || (strcmp(s->label, ln)==0 && (b = 1)) ) {
 		     char * lr = (b ? ll : ln);
 		     for (t=to->stns; t; t=t->next) {
 			if (strcmp(t->label,lr)==0) {
-			   printf(msg(/*Breaking survey at leg %s -> %s*/611), s->label, t->label);
+			   printf(msg(/*Breaking survey loop at leg %s -> %s*/518), s->label, t->label);
 			   putnl();
-			   l->fDone = (b ? BREAK_TO : BREAK_FR);
+			   l->broken = (b ? BREAK_TO : BREAK_FR);
 			   goto loopend;
 			}
 		     }
@@ -406,15 +414,16 @@ parseconfigline(char *ln)
 	       }
 	    }
 	 }
-	 warning(/*Failed to find leg %s-%s in config, line %i*/606, ll, ln, lineno);
+	 warning_in_file(fnm, lineno, /*Failed to find leg %s -> %s*/511, ll, ln);
       }
    } else {
-      fatalerror(/*Don't understand command `%s' in config, line %i*/600, ln, lineno);
+      fatalerror_in_file(fnm, lineno, /*Unknown command `%s'*/12, ln);
    }
  loopend:
    ln = delimword(lc, &lc);
    if (*ln != 0) {
-      fatalerror(/*Unexpected content `%s' in config, line %i*/601, ln, lineno);
+      fatalerror_in_file(fnm, lineno, /*End of line not blank*/15);
+      /* FIXME: give ln as context? */
    }
 }
 
@@ -448,6 +457,7 @@ main(int argc, char **argv)
    point *p;
    const char *survey = NULL;
    const char *specfile = NULL;
+   img *pimg;
 
    msg_init(argv);
 
@@ -463,7 +473,13 @@ main(int argc, char **argv)
    if (argv[optind]) {
       fnm_out = argv[optind];
    } else {
-      fnm_out = add_ext("extend", EXT_SVX_3D);
+      char * base_in = base_from_fnm(fnm_in);
+      char * base_out = osmalloc(strlen(base_in) + 8);
+      strcpy(base_out, base_in);
+      strcat(base_out, "_extend");
+      fnm_out = add_ext(base_out, EXT_SVX_3D);
+      osfree(base_in);
+      osfree(base_out);
    }
 
    /* try to open image file, and check it has correct header */
@@ -513,26 +529,24 @@ main(int argc, char **argv)
 
    if (specfile) {
       FILE *fs = NULL;
-      printf(msg(/*Applying specfile: `%s'*/613), specfile);
+      char *fnm_used;
+      printf(msg(/*Applying specfile: `%s'*/521), specfile);
       putnl();
-      fs = fopenWithPthAndExt("", specfile, NULL, "r", NULL);
+      fs = fopenWithPthAndExt("", specfile, NULL, "r", &fnm_used);
       if (fs == NULL) fatalerror(/*Unable to open file*/93, specfile);
       while (!feof(fs)) {
-	 char *lbuf = NULL;
-	 lbuf = getline_alloc(fs, 32);
+	 char *lbuf = getline_alloc(fs, 32);
 	 lineno++;
-	 if (!lbuf) {
-	    error(/*Error reading line %i from spec file*/612, lineno);
-	    osfree(lbuf);
-	    break;
-	 }
-	 parseconfigline(lbuf);
+	 if (!lbuf)
+	    fatalerror_in_file(fnm_used, lineno, /*Error reading file*/18);
+	 parseconfigline(fnm_used, lbuf);
 	 osfree(lbuf);
       }
+      osfree(fnm_used);
    }
 
    if (start == NULL) { /* i.e. start wasn't specified in specfile */
-   
+
       /* start at the highest entrance with some legs attached */
       for (p = headpoint.next; p != NULL; p = p->next) {
 	 if (p->order > 0 && p->p.z > zMax) {
@@ -576,12 +590,13 @@ main(int argc, char **argv)
       }
    }
 
-   printf(msg(/*Writing out .3d file...*/614));
+   printf(msg(/*Writing %s...*/522), fnm_out);
    putnl();
-   pimg = img_open_write(fnm_out, desc, fTrue);
+   pimg_out = img_open_write(fnm_out, desc, fTrue);
 
-   do_stn(start, 0.0, NULL, ERIGHT); /* only does single connected component currently */
-   if (!img_close(pimg)) {
+   /* Only does single connected component currently. */
+   do_stn(start, 0.0, NULL, ERIGHT, 0);
+   if (!img_close(pimg_out)) {
       (void)remove(fnm_out);
       fatalerror(img_error(), fnm_out);
    }
@@ -589,106 +604,112 @@ main(int argc, char **argv)
    return EXIT_SUCCESS;
 }
 
+static int adjust_direction(int dir, int by) {
+    if (by == ESWAP)
+	return dir ^ (ELEFT|ERIGHT);
+    if (by)
+	return by;
+    return dir;
+}
+
 static void
-do_stn(point *p, double X, const char *prefix, int dir)
+do_stn(point *p, double X, const char *prefix, int dir, int labOnly)
 {
    leg *l, *lp;
    double dX;
    const stn *s;
+   int odir = dir;
 
    for (s = p->stns; s; s = s->next) {
-      img_write_item(pimg, img_LABEL, s->flags, s->label, X, 0, p->p.z);
+      img_write_item(pimg_out, img_LABEL, s->flags, s->label, X, 0, p->p.z);
    }
-   if (p->fDone & BREAK) {
-     return;
+   if (labOnly || p->fBroken) {
+      return;
    }
 
    lp = &headleg;
    for (l = lp->next; l; lp = l, l = lp->next) {
-      if (l->fDone & DONE) {
+      dir = odir;
+      if (l->fDone) {
 	 /* this case happens if a recursive call causes the next leg to be
 	  * removed, leaving our next pointing to a leg which has been dealt
 	  * with... */
-      } else if (l->prefix == prefix) {
-	 if (l->to == p) {
-	    if (l->fDone & BREAK_TO) continue;
-	    lp->next = l->next;
-	    /* adjust direction of extension if necessary */
-	    if (l->to->fDir == ESWAP)               dir = (dir==ERIGHT ? ELEFT : ERIGHT);
-	    else if (l->to->fDir & (ERIGHT|ELEFT))  dir = l->to->fDir;
-	    if (l->fDir == ESWAP)                   dir = (dir==ERIGHT ? ELEFT : ERIGHT);
-	    else if (l->fDir & (ERIGHT|ELEFT))      dir = l->fDir;
+	 continue;
+      }
+      if (l->prefix != prefix) {
+	 continue;
+      }
+      if (l->to == p) {
+	 if (l->broken & BREAK_TO) continue;
+	 lp->next = l->next;
+	 /* adjust direction of extension if necessary */
+	 dir = adjust_direction(dir, l->to->dir);
+	 dir = adjust_direction(dir, l->dir);
 
-	    dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
-	    if (dir == ELEFT) dX *= -1.0;
-	    img_write_item(pimg, img_MOVE, 0, NULL, X + dX, 0, l->fr->p.z);
-	    img_write_item(pimg, img_LINE, l->flags, l->prefix,
-			   X, 0, l->to->p.z);
-	    l->fDone |= DONE;
-	    do_stn(l->fr, X + dX, l->prefix, dir);
-	    l = lp;
-	 } else if (l->fr == p) {
-	    if (l->fDone & BREAK_FR) continue;
-	    lp->next = l->next;
-	    /* adjust direction of extension if necessary */
-	    if (l->fr->fDir == ESWAP)               dir = (dir==ERIGHT ? ELEFT : ERIGHT);
-	    else if (l->fr->fDir & (ERIGHT|ELEFT))  dir = l->fr->fDir;
-	    if (l->fDir == ESWAP)                   dir = (dir==ERIGHT ? ELEFT : ERIGHT);
-	    else if (l->fDir & (ERIGHT|ELEFT))      dir = l->fDir;
+	 dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
+	 if (dir == ELEFT) dX = -dX;
+	 img_write_item(pimg_out, img_MOVE, 0, NULL, X + dX, 0, l->fr->p.z);
+	 img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
+			X, 0, l->to->p.z);
+	 l->fDone = 1;
+	 do_stn(l->fr, X + dX, l->prefix, dir, (l->broken & BREAK_FR));
+	 l = lp;
+      } else if (l->fr == p) {
+	 if (l->broken & BREAK_FR) continue;
+	 lp->next = l->next;
+	 /* adjust direction of extension if necessary */
+	 dir = adjust_direction(dir, l->fr->dir);
+	 dir = adjust_direction(dir, l->dir);
 
-	    dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
-	    if (dir == ELEFT) dX *= -1.0;
-	    img_write_item(pimg, img_MOVE, 0, NULL, X, 0, l->fr->p.z);
-	    img_write_item(pimg, img_LINE, l->flags, l->prefix,
-			   X + dX, 0, l->to->p.z);
-	    l->fDone |= DONE;
-	    do_stn(l->to, X + dX, l->prefix, dir);
-	    l = lp;
-	 }
+	 dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
+	 if (dir == ELEFT) dX = -dX;
+	 img_write_item(pimg_out, img_MOVE, 0, NULL, X, 0, l->fr->p.z);
+	 img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
+			X + dX, 0, l->to->p.z);
+	 l->fDone = 1;
+	 do_stn(l->to, X + dX, l->prefix, dir, (l->broken & BREAK_TO));
+	 l = lp;
       }
    }
    lp = &headleg;
    for (l = lp->next; l; lp = l, l = lp->next) {
-      if (l->fDone & DONE) {
+      dir = odir;
+      if (l->fDone) {
 	 /* this case happens iff a recursive call causes the next leg to be
 	  * removed, leaving our next pointing to a leg which has been dealt
 	  * with... */
-      } else {
-	 if (l->to == p) {
-	    if (l->fDone & BREAK_TO) continue;
-	    lp->next = l->next;
-	    /* adjust direction of extension if necessary */
-	    if (l->to->fDir == ESWAP)               dir = (dir==ERIGHT ? ELEFT : ERIGHT);
-	    else if (l->to->fDir & (ERIGHT|ELEFT))  dir = l->to->fDir;
-	    if (l->fDir == ESWAP)                   dir = (dir==ERIGHT ? ELEFT : ERIGHT);
-	    else if (l->fDir & (ERIGHT|ELEFT))      dir = l->fDir;
-	    
-	    dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
-	    if (dir == ELEFT) dX *= -1.0;
-	    img_write_item(pimg, img_MOVE, 0, NULL, X + dX, 0, l->fr->p.z);
-	    img_write_item(pimg, img_LINE, l->flags, l->prefix,
-			   X, 0, l->to->p.z);
-	    l->fDone |= DONE;
-	    do_stn(l->fr, X + dX, l->prefix, dir);
-	    l = lp;
-	 } else if (l->fr == p) {
-	    if (l->fDone & BREAK_FR) continue;
-	    lp->next = l->next;
-	    /* adjust direction of extension if necessary */
-	    if (l->fr->fDir == ESWAP)               dir = (dir==ERIGHT ? ELEFT : ERIGHT);
-	    else if (l->fr->fDir & (ERIGHT|ELEFT))  dir = l->fr->fDir;
-	    if (l->fDir == ESWAP)                   dir = (dir==ERIGHT ? ELEFT : ERIGHT);
-	    else if (l->fDir & (ERIGHT|ELEFT))      dir = l->fDir;
+	 continue;
+      }
+      if (l->to == p) {
+	 if (l->broken & BREAK_TO) continue;
+	 lp->next = l->next;
+	 /* adjust direction of extension if necessary */
+	 dir = adjust_direction(dir, l->to->dir);
+	 dir = adjust_direction(dir, l->dir);
 
-	    dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
-	    if (dir == ELEFT) dX *= -1.0;
-	    img_write_item(pimg, img_MOVE, 0, NULL, X, 0, l->fr->p.z);
-	    img_write_item(pimg, img_LINE, l->flags, l->prefix,
-			   X + dX, 0, l->to->p.z);
-	    l->fDone |= DONE;
-	    do_stn(l->to, X + dX, l->prefix, dir);
-	    l = lp;
-	 }
+	 dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
+	 if (dir == ELEFT) dX = -dX;
+	 img_write_item(pimg_out, img_MOVE, 0, NULL, X + dX, 0, l->fr->p.z);
+	 img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
+			X, 0, l->to->p.z);
+	 l->fDone = 1;
+	 do_stn(l->fr, X + dX, l->prefix, dir, (l->broken & BREAK_FR));
+	 l = lp;
+      } else if (l->fr == p) {
+	 if (l->broken & BREAK_FR) continue;
+	 lp->next = l->next;
+	 /* adjust direction of extension if necessary */
+	 dir = adjust_direction(dir, l->fr->dir);
+	 dir = adjust_direction(dir, l->dir);
+
+	 dX = hypot(l->fr->p.x - l->to->p.x, l->fr->p.y - l->to->p.y);
+	 if (dir == ELEFT) dX = -dX;
+	 img_write_item(pimg_out, img_MOVE, 0, NULL, X, 0, l->fr->p.z);
+	 img_write_item(pimg_out, img_LINE, l->flags, l->prefix,
+			X + dX, 0, l->to->p.z);
+	 l->fDone = 1;
+	 do_stn(l->to, X + dX, l->prefix, dir, (l->broken & BREAK_TO));
+	 l = lp;
       }
    }
 }
