@@ -50,13 +50,16 @@
 #define PLAN 1
 #define ELEVATION 2
 
+// How many bins per letter height to use when working out non-overlapping
+// labels.
+const unsigned int QUANTISE_FACTOR = 2;
+
 #ifdef _WIN32
 static const int FONT_SIZE = 8;
 #else
 static const int FONT_SIZE = 9;
 #endif
 static const int CROSS_SIZE = 3;
-static const Double COMPASS_SIZE = 24.0;
 static const int COMPASS_OFFSET_X = 60;
 static const int COMPASS_OFFSET_Y = 80;
 static const int INDICATOR_BOX_SIZE = 60;
@@ -65,7 +68,7 @@ static const int INDICATOR_MARGIN = 5;
 static const int INDICATOR_OFFSET_X = 15;
 static const int INDICATOR_OFFSET_Y = 15;
 static const int CLINO_OFFSET_X = 6 + INDICATOR_OFFSET_X +
-                                  INDICATOR_BOX_SIZE + INDICATOR_GAP;
+				  INDICATOR_BOX_SIZE + INDICATOR_GAP;
 static const int DEPTH_BAR_OFFSET_X = 16;
 static const int DEPTH_BAR_EXTRA_LEFT_MARGIN = 2;
 static const int DEPTH_BAR_BLOCK_WIDTH = 20;
@@ -113,7 +116,7 @@ BEGIN_EVENT_TABLE(GfxCore, wxWindow)
 END_EVENT_TABLE()
 
 GfxCore::GfxCore(MainFrm* parent, wxWindow* parent_win) :
-    wxWindow(parent_win, 100),
+    wxWindow(parent_win, 100, wxDefaultPosition, wxDefaultSize, wxWANTS_CHARS),
     m_Font(FONT_SIZE, wxSWISS, wxNORMAL, wxNORMAL, FALSE, "Helvetica",
 	   wxFONTENCODING_ISO8859_1),
     m_InitialisePending(false)
@@ -175,7 +178,7 @@ GfxCore::~GfxCore()
     TryToFreeArrays();
 
     if (m_OffscreenBitmap) {
-        delete m_OffscreenBitmap;
+	delete m_OffscreenBitmap;
     }
 
     DELETE_ARRAY(m_Pens);
@@ -696,9 +699,9 @@ void GfxCore::RedrawOffscreen()
 			 m_00, m_01, m_02, m_20, m_21, m_22);
 	    }
 	}
-	    
-	// Draw surface legs.
+
 	if (m_Surface) {
+	    // Draw the surface legs.
 	    int start;
 	    int end;
 	    int inc;
@@ -813,7 +816,7 @@ void GfxCore::RedrawOffscreen()
     }
 
     m_DrawDC.EndDrawing();
-    
+
     drawtime = timer.Time();
 }
 
@@ -853,7 +856,7 @@ void GfxCore::OnPaint(wxPaintEvent&)
 
 	iter++;
     }
-    
+
     if (!m_Rotating && !m_SwitchingTo) {
 	int here_x = INT_MAX, here_y = INT_MAX;
 	// Draw "here" and "there".
@@ -1104,7 +1107,7 @@ void GfxCore::Draw2dIndicators()
 	    str = wxString::Format("%03d", int(m_PanAngle * 180.0 / M_PI));
 	} else {
 	    str = wxString::Format("%03d", int(m_PanAngle * 200.0 / M_PI));
-	}	
+	}
 	m_DrawDC.GetTextExtent(str, &w, &h);
 	m_DrawDC.DrawText(str, pan_centre_x + width / 2 - w, height);
 	str = wxString(msg(/*Facing*/203));
@@ -1118,7 +1121,7 @@ void GfxCore::Draw2dIndicators()
 	    angle = int(-m_TiltAngle * 180.0 / M_PI);
 	} else {
 	    angle = int(-m_TiltAngle * 200.0 / M_PI);
-	}	
+	}
 	str = angle ? wxString::Format("%+03d", angle) : wxString("00");
 	m_DrawDC.GetTextExtent(str, &w, &h);
 	m_DrawDC.DrawText(str, elev_centre_x + width / 2 - w, height);
@@ -1128,35 +1131,6 @@ void GfxCore::Draw2dIndicators()
     }
 }
 
-// FIXME: either remove this, or make it an option...
-void GfxCore::DrawCompass()
-{
-    // Draw the 3d compass.
-
-    wxPoint pt[3];
-
-    SetColour(col_TURQUOISE);
-    m_DrawDC.DrawLine(CompassPtToScreen(0.0, 0.0, -COMPASS_SIZE),
-		      CompassPtToScreen(0.0, 0.0, COMPASS_SIZE));
-
-    pt[0] = CompassPtToScreen(-COMPASS_SIZE / 3.0f, 0.0, -COMPASS_SIZE * 2.0f / 3.0f);
-    pt[1] = CompassPtToScreen(0.0, 0.0, -COMPASS_SIZE);
-    pt[2] = CompassPtToScreen(COMPASS_SIZE / 3.0f, 0.0, -COMPASS_SIZE * 2.0f / 3.0f);
-    m_DrawDC.DrawLines(3, pt);
-
-    m_DrawDC.DrawLine(CompassPtToScreen(-COMPASS_SIZE, 0.0, 0.0),
-		      CompassPtToScreen(COMPASS_SIZE, 0.0, 0.0));
-
-    SetColour(col_GREEN);
-    m_DrawDC.DrawLine(CompassPtToScreen(0.0, -COMPASS_SIZE, 0.0),
-		      CompassPtToScreen(0.0, COMPASS_SIZE, 0.0));
-
-    pt[0] = CompassPtToScreen(-COMPASS_SIZE / 3.0f, -COMPASS_SIZE * 2.0f / 3.0f, 0.0);
-    pt[1] = CompassPtToScreen(0.0, -COMPASS_SIZE, 0.0);
-    pt[2] = CompassPtToScreen(COMPASS_SIZE / 3.0f, -COMPASS_SIZE * 2.0f / 3.0f, 0.0);
-    m_DrawDC.DrawLines(3, pt);
-}
-
 void GfxCore::DrawNames()
 {
     // Draw station names.
@@ -1174,8 +1148,7 @@ void GfxCore::NattyDrawNames()
 {
     // Draw station names, without overlapping.
     // FIXME: copied to OnSize()
-    const int dv = 2;
-    const int quantise = int(FONT_SIZE / dv);
+    const int quantise(FONT_SIZE / QUANTISE_FACTOR);
     const int quantised_x = m_XSize / quantise;
     const int quantised_y = m_YSize / quantise;
     const size_t buffer_size = quantised_x * quantised_y;
@@ -1184,6 +1157,14 @@ void GfxCore::NattyDrawNames()
 
     list<LabelInfo*>::const_iterator label = m_Parent->GetLabels();
     for ( ; label != m_Parent->GetLabelsEnd(); ++label) {
+	if (!((m_Surface && (*label)->IsSurface()) ||
+	      (m_Legs && (*label)->IsUnderground()) ||
+	      (!(*label)->IsSurface() && !(*label)->IsUnderground()))) {
+	    // if this station isn't to be displayed, skip to the next
+	    // (last case is for stns with no legs attached)
+	    continue;
+	}
+
 	Double x = GridXToScreen((*label)->x, (*label)->y, (*label)->z);
 	Double y = GridYToScreen((*label)->x, (*label)->y, (*label)->z)
 	    + CROSS_SIZE - FONT_SIZE;
@@ -1201,7 +1182,7 @@ void GfxCore::NattyDrawNames()
 
 	if (ix >= 0 && ix < quantised_x && iy >= 0 && iy < quantised_y) {
 	    char * test = &m_LabelGrid[ix + iy * quantised_x];
-	    int len = str.Length() * dv + 1;
+	    int len = str.Length() * QUANTISE_FACTOR + 1;
 	    reject = (ix + len >= quantised_x);
 	    int i = 0;
 	    while (!reject && i++ < len) {
@@ -1251,7 +1232,7 @@ void GfxCore::DrawDepthbar()
 		/ (m_Bands - 1);
 
 	strs[band] = FormatLength(z, false);
-	
+
 	int x, dummy;
 	m_DrawDC.GetTextExtent(strs[band], &x, &dummy);
 	if (x > size) size = x;
@@ -1263,7 +1244,7 @@ void GfxCore::DrawDepthbar()
     SetColour(col_BLACK);
     SetColour(col_DARK_GREY, true);
     m_DrawDC.DrawRectangle(x_min - DEPTH_BAR_MARGIN
-		    	     - DEPTH_BAR_EXTRA_LEFT_MARGIN,
+			     - DEPTH_BAR_EXTRA_LEFT_MARGIN,
 			   DEPTH_BAR_OFFSET_Y - DEPTH_BAR_MARGIN*2,
 			   DEPTH_BAR_BLOCK_WIDTH + size + DEPTH_BAR_MARGIN*3 +
 			     DEPTH_BAR_EXTRA_LEFT_MARGIN,
@@ -1274,7 +1255,7 @@ void GfxCore::DrawDepthbar()
 	    m_DrawDC.SetPen(m_Parent->GetPen(band));
 	    m_DrawDC.SetBrush(m_Parent->GetBrush(band));
 	    m_DrawDC.DrawRectangle(x_min,
-			           y - DEPTH_BAR_BLOCK_HEIGHT,
+				   y - DEPTH_BAR_BLOCK_HEIGHT,
 				   DEPTH_BAR_BLOCK_WIDTH,
 				   DEPTH_BAR_BLOCK_HEIGHT);
 	}
@@ -1426,6 +1407,7 @@ void GfxCore::DrawScalebar()
 
 void GfxCore::OnLButtonDown(wxMouseEvent& event)
 {
+    SetFocus();
     if (m_PlotData && m_Lock != lock_POINT) {
 	m_DraggingLeft = true;
 	m_ScaleBar.drag_start_offset_x = m_ScaleBar.offset_x;
@@ -1458,6 +1440,7 @@ void GfxCore::OnLButtonUp(wxMouseEvent& event)
 
 void GfxCore::OnMButtonDown(wxMouseEvent& event)
 {
+    SetFocus();
     if (m_PlotData && m_Lock == lock_NONE) {
 	m_DraggingMiddle = true;
 	m_DragStart = wxPoint(event.GetX(), event.GetY());
@@ -1476,6 +1459,7 @@ void GfxCore::OnMButtonUp(wxMouseEvent&)
 
 void GfxCore::OnRButtonDown(wxMouseEvent& event)
 {
+    SetFocus();
     if (m_PlotData) {
 	m_DragStart = wxPoint(event.GetX(), event.GetY());
 	m_ScaleBar.drag_start_offset_x = m_ScaleBar.offset_x;
@@ -1580,7 +1564,7 @@ void GfxCore::HandleTranslate(wxPoint point)
     // Handle a mouse movement during translation mode.
     int dx = point.x - m_DragStart.x;
     int dy = point.y - m_DragStart.y;
-    
+
     if (m_ReverseControls) {
 	dx = -dx;
 	dy = -dy;
@@ -1637,13 +1621,13 @@ void GfxCore::CheckHitTestGrid(wxPoint& point, bool centre)
 
 	ds += dy * dy;
 	if (ds >= dist_sqrd) continue;
- 
+
 	dist_sqrd = ds;
 	best = pt;
-	
+
 	if (ds == 0) break;
     }
-    
+
     if (best) {
 	m_Parent->SetMouseOverStation(best);
 	if (centre) {
@@ -1814,8 +1798,7 @@ void GfxCore::OnSize(wxSizeEvent& event)
 
     if (m_DoneFirstShow) {
 	// FIXME: copied from NattyDrawNames()
-	const int dv = 2;
-	const int quantise = int(FONT_SIZE / dv);
+	const int quantise(FONT_SIZE / QUANTISE_FACTOR);
 	const int quantised_x = m_XSize / quantise;
 	const int quantised_y = m_YSize / quantise;
 	const size_t buffer_size = quantised_x * quantised_y;
@@ -2114,7 +2097,7 @@ void GfxCore::OnElevation()
 	    TiltCave(-m_TiltAngle);
 	    m_SwitchingTo = 0;
 	    ForceRefresh();
-    } 
+    }
 }
 
 void GfxCore::OnElevationUpdate(wxUpdateUIEvent& cmd)
@@ -2567,9 +2550,9 @@ void GfxCore::CreateHitTestGrid()
 
 	// Calculate screen coordinates.
 	int cx = (int)GridXToScreen(label->GetX(), label->GetY(), label->GetZ());
- 	if (cx < 0 || cx >= m_XSize) continue;
+	if (cx < 0 || cx >= m_XSize) continue;
 	int cy = (int)GridYToScreen(label->GetX(), label->GetY(), label->GetZ());
- 	if (cy < 0 || cy >= m_YSize) continue;
+	if (cy < 0 || cy >= m_YSize) continue;
 
 	// On-screen, so add to hit-test grid...
 	int grid_x = (cx * (HITTEST_SIZE - 1)) / m_XSize;
@@ -2704,7 +2687,7 @@ void GfxCore::OnKeyPress(wxKeyEvent &e)
 	default:
 	    e.Skip();
     }
- 
+
     // OnPaint clears m_RedrawOffscreen so it'll only still be set if we need
     // to redraw
     if (m_RedrawOffscreen) ForceRefresh();
@@ -2716,7 +2699,7 @@ GfxCore::OnPrint(const wxString &filename, const wxString &title,
 {
     svxPrintDlg * p;
     p = new svxPrintDlg(m_Parent, filename, title, datestamp,
-			m_PanAngle, m_TiltAngle,
+			deg(m_PanAngle), deg(m_TiltAngle),
 			m_Names, m_Crosses, m_Legs, m_Surface);
     p->Show(TRUE);
 }
