@@ -1,6 +1,6 @@
 /* img.h
  * Header file for routines to read and write Survex ".3d" image files
- * Copyright (C) Olly Betts 1993,1994,1997,2001,2002
+ * Copyright (C) Olly Betts 1993,1994,1997,2001,2002,2003,2004,2005
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ extern "C" {
 #endif
 
 #include <stdio.h>
+#include <time.h> /* for time_t */
 
 # ifdef IMG_HOSTED
 #  include "useful.h"
@@ -38,6 +39,7 @@ extern "C" {
  * Put crosses where labels are. */
 /* # define img_CROSS  2 */
 # define img_LABEL  3
+# define img_XSECT  4
 
 # define img_FLAG_SURFACE   0x01
 # define img_FLAG_DUPLICATE 0x02
@@ -49,6 +51,9 @@ extern "C" {
 # define img_SFLAG_EXPORTED    0x08
 # define img_SFLAG_FIXED       0x10
 
+# define img_XFLAG_END      0x01
+
+/* 3D coordinates (in metres) */
 typedef struct {
    double x, y, z;
 } img_point;
@@ -59,6 +64,10 @@ typedef struct {
    int flags;
    char *title;
    char *datestamp;
+   char separator; /* character used to separate survey levels ('.' usually) */
+   time_t date1, date2;
+   double l, r, u, d;
+   char * filename_opened; /* The filename actually opened (e.g. may have ".3d" added). */
    /* all other members are for internal use only */
    FILE *fh;          /* file handle of image file */
    char *label_buf;
@@ -79,12 +88,15 @@ typedef struct {
     *   1 => 0.01 binary,
     *   2 => byte actions and flags
     *   3 => prefixes for legs; compressed prefixes
+    *   4 => survey date
+    *   5 => LRUD info
     */
    int version;
    char *survey;
    size_t survey_len;
    int pending; /* for old style text format files and survey filtering */
    img_point mv;
+   time_t olddate1, olddate2;
 } img;
 
 /* Which version of the file format to output (defaults to newest) */
@@ -107,9 +119,8 @@ img *img_open_survey(const char *fnm, const char *survey);
 /* Open a .3d file for output
  * fnm is the filename
  * title_buf is the title
- * fBinary == 0 for ASCII .3d file
- *         != 0 for binary .3d file
- * NB only the original .3d file format has an ASCII variant
+ * fBinary is ignored (it used to select an ASCII variant of the earliest
+ * version of the 3d file format)
  * Returns pointer to an img struct or NULL for error (check img_error()
  * for details)
  */
@@ -130,7 +141,8 @@ int img_read_item(img *pimg, img_point *p);
 /* Write a item to a .3d file
  * pimg is a pointer to an img struct returned by img_open_write()
  * code is one of the img_XXXX #define-d above
- * flags is the leg or station flags (meaningful for img_LINE and img_LABEL
+ * flags is the leg, station, or xsect flags
+ * (meaningful for img_LINE, img_LABEL, and img_XSECT respectively)
  * s is the label (only meaningful for img_LABEL)
  * x, y, z are the coordinates
  */
