@@ -47,10 +47,6 @@ using namespace std;
 
 const int NUM_DEPTH_COLOURS = 13; // up to 13
 
-#define TOOLBAR_BITMAP(file) wxBitmap(wxString(msg_cfgpth()) + \
-    wxCONFIG_PATH_SEPARATOR + wxString("icons") + wxCONFIG_PATH_SEPARATOR + \
-    wxString(file), wxBITMAP_TYPE_PNG)
-
 #include "avenpal.h"
 
 class AvenSplitterWindow : public wxSplitterWindow {
@@ -263,10 +259,17 @@ MainFrm::MainFrm(const wxString& title, const wxPoint& pos, const wxSize& size)
     wxFrame(NULL, 101, title, pos, size, wxDEFAULT_FRAME_STYLE | wxNO_FULL_REPAINT_ON_RESIZE),
     m_Gfx(NULL), m_NumEntrances(0), m_NumFixedPts(0), m_NumExportedPts(0)
 {
+    icon_path = msg_cfgpth();
+    icon_path += wxCONFIG_PATH_SEPARATOR;
+    icon_path += "icons";
+    icon_path += wxCONFIG_PATH_SEPARATOR;
+
 #ifdef _WIN32
     // The peculiar name is so that the icon is the first in the file
     // (required by Microsoft Windows for this type of icon)
     SetIcon(wxIcon("aaaaaAven"));
+#else
+    SetIcon(wxIcon(icon_path + "aven.png", wxBITMAP_TYPE_PNG));
 #endif
 
     InitialisePensAndBrushes();
@@ -316,7 +319,7 @@ void MainFrm::CreateMenuBar()
     filemenu->Append(menu_FILE_PRINT, GetTabMsg(/*@Print...##Ctrl+P*/380));
     filemenu->Append(menu_FILE_PAGE_SETUP, GetTabMsg(/*P@age Setup...*/381));
     filemenu->AppendSeparator();
-    filemenu->Append(menu_FILE_EXPORT, "Export as..."); // FIXME TRANSLATE
+    filemenu->Append(menu_FILE_EXPORT, GetTabMsg(/*@Export as...*/382));
     filemenu->AppendSeparator();
     filemenu->Append(menu_FILE_QUIT, GetTabMsg(/*@Quit##Ctrl+Q*/221));
 
@@ -403,40 +406,43 @@ void MainFrm::CreateMenuBar()
     SetMenuBar(menubar);
 }
 
+// ICON must be a literal string.
+#define TOOLBAR_BITMAP(ICON) wxBitmap(icon_path + ICON".png", wxBITMAP_TYPE_PNG)
+
 void MainFrm::CreateToolBar()
 {
     // Create the toolbar.
 
     wxToolBar* toolbar = wxFrame::CreateToolBar();
 
-#ifndef _WIN32
+#ifndef __WXGTK20__
     toolbar->SetMargins(5, 5);
 #endif
 
-    toolbar->AddTool(menu_FILE_OPEN, TOOLBAR_BITMAP("open.png"), "Open a 3D file for viewing");
+    toolbar->AddTool(menu_FILE_OPEN, TOOLBAR_BITMAP("open"), "Open a 3D file for viewing");
     toolbar->AddSeparator();
-    toolbar->AddTool(menu_ROTATION_TOGGLE, TOOLBAR_BITMAP("rotation.png"),
+    toolbar->AddTool(menu_ROTATION_TOGGLE, TOOLBAR_BITMAP("rotation"),
 		     wxNullBitmap, true, -1, -1, NULL, "Toggle rotation");
     toolbar->AddSeparator();
-    toolbar->AddTool(menu_ORIENT_PLAN, TOOLBAR_BITMAP("plan.png"), "Switch to plan view");
-    toolbar->AddTool(menu_ORIENT_ELEVATION, TOOLBAR_BITMAP("elevation.png"), "Switch to elevation view");
+    toolbar->AddTool(menu_ORIENT_PLAN, TOOLBAR_BITMAP("plan"), "Switch to plan view");
+    toolbar->AddTool(menu_ORIENT_ELEVATION, TOOLBAR_BITMAP("elevation"), "Switch to elevation view");
     toolbar->AddSeparator();
-    toolbar->AddTool(menu_ORIENT_DEFAULTS, TOOLBAR_BITMAP("defaults.png"), "Restore default view");
+    toolbar->AddTool(menu_ORIENT_DEFAULTS, TOOLBAR_BITMAP("defaults"), "Restore default view");
     toolbar->AddSeparator();
-    toolbar->AddTool(menu_VIEW_SHOW_NAMES, TOOLBAR_BITMAP("names.png"), wxNullBitmap, true,
+    toolbar->AddTool(menu_VIEW_SHOW_NAMES, TOOLBAR_BITMAP("names"), wxNullBitmap, true,
 		     -1, -1, NULL, "Show station names");
-    toolbar->AddTool(menu_VIEW_SHOW_CROSSES, TOOLBAR_BITMAP("crosses.png"), wxNullBitmap, true,
+    toolbar->AddTool(menu_VIEW_SHOW_CROSSES, TOOLBAR_BITMAP("crosses"), wxNullBitmap, true,
 		     -1, -1, NULL, "Show crosses on stations");
-    toolbar->AddTool(menu_VIEW_SHOW_ENTRANCES, TOOLBAR_BITMAP("entrances.png"), wxNullBitmap, true,
+    toolbar->AddTool(menu_VIEW_SHOW_ENTRANCES, TOOLBAR_BITMAP("entrances"), wxNullBitmap, true,
 		     -1, -1, NULL, "Highlight entrances");
-    toolbar->AddTool(menu_VIEW_SHOW_FIXED_PTS, TOOLBAR_BITMAP("fixed-pts.png"), wxNullBitmap, true,
+    toolbar->AddTool(menu_VIEW_SHOW_FIXED_PTS, TOOLBAR_BITMAP("fixed-pts"), wxNullBitmap, true,
 		     -1, -1, NULL, "Highlight fixed points");
-    toolbar->AddTool(menu_VIEW_SHOW_EXPORTED_PTS, TOOLBAR_BITMAP("exported-pts.png"), wxNullBitmap, true,
+    toolbar->AddTool(menu_VIEW_SHOW_EXPORTED_PTS, TOOLBAR_BITMAP("exported-pts"), wxNullBitmap, true,
 		     -1, -1, NULL, "Highlight exported stations");
     toolbar->AddSeparator();
-    toolbar->AddTool(menu_VIEW_SHOW_LEGS, TOOLBAR_BITMAP("ug-legs.png"), wxNullBitmap, true,
+    toolbar->AddTool(menu_VIEW_SHOW_LEGS, TOOLBAR_BITMAP("ug-legs"), wxNullBitmap, true,
 		     -1, -1, NULL, "Show underground surveys");
-    toolbar->AddTool(menu_VIEW_SHOW_SURFACE, TOOLBAR_BITMAP("surface-legs.png"), wxNullBitmap, true,
+    toolbar->AddTool(menu_VIEW_SHOW_SURFACE, TOOLBAR_BITMAP("surface-legs"), wxNullBitmap, true,
 		     -1, -1, NULL, "Show surface surveys");
     toolbar->AddSeparator();
 
@@ -550,12 +556,13 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
     // chop legs such that no legs cross depth colour boundaries and prepare
     // the data for drawing.
 
-    // Load the survey data.
 #if 0
     wxStopWatch timer;
     timer.Start();
 #endif
 
+    // Load the survey data.
+
     img* survey = img_open_survey(file, prefix.c_str());
     if (!survey) {
 	wxString m = wxString::Format(msg(img_error()), file.c_str());
@@ -563,13 +570,10 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 	return false;
     }
 
-    m_File = file;
+    m_File = survey->filename_opened;
 
     m_Tree->DeleteAllItems();
 
-    m_TreeRoot = m_Tree->AddRoot(wxFileNameFromPath(file));
-    m_Tree->SetEnabled();
-
     // Create a list of all the leg vertices, counting them and finding the
     // extent of the survey at the same time.
 
@@ -630,8 +634,7 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		break;
 	    }
 
-	    case img_LABEL:
-	    {
+	    case img_LABEL: {
 		LabelInfo* label = new LabelInfo;
 		label->text = survey->label;
 		label->x = pt.x;
@@ -656,8 +659,7 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 		break;
 	    }
 
-	    case img_BAD:
-	    {
+	    case img_BAD: {
 		m_Labels.clear();
 
 		// FIXME: Do we need to reset all these? - Olly
@@ -730,7 +732,6 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 
     // Fill the tree of stations and prefixes.
     FillTree();
-    m_Tree->Expand(m_TreeRoot);
 
     // Sort labels so that entrances are displayed in preference,
     // then fixed points, then exported points, then other points.
@@ -751,21 +752,22 @@ bool MainFrm::LoadData(const wxString& file, wxString prefix)
 #endif
 
     // Update window title.
-    SetTitle(wxString(APP_NAME" - [") + file + wxString("]"));
-    m_File = file;
+    SetTitle(wxString(APP_NAME" - [") + m_File + wxString("]"));
 
     return true;
 }
 
 void MainFrm::FillTree()
 {
-    // Fill the tree of stations and prefixes.
+    // Create the root of the tree.
+    wxTreeItemId treeroot = m_Tree->AddRoot(wxFileNameFromPath(m_File));
 
-    list<LabelInfo*>::iterator pos = m_Labels.begin();
+    // Fill the tree of stations and prefixes.
     stack<wxTreeItemId> previous_ids;
     wxString current_prefix = "";
-    wxTreeItemId current_id = m_TreeRoot;
+    wxTreeItemId current_id = treeroot;
 
+    list<LabelInfo*>::iterator pos = m_Labels.begin();
     while (pos != m_Labels.end()) {
 	LabelInfo* label = *pos++;
 
@@ -777,12 +779,12 @@ void MainFrm::FillTree()
 	    // no need to fiddle with branches...
 	}
 	// If not, then see if we've descended to a new prefix.
-	else if (prefix.Length() > current_prefix.Length() &&
+	else if (prefix.length() > current_prefix.length() &&
 		 prefix.StartsWith(current_prefix) &&
-		 (prefix[current_prefix.Length()] == separator ||
+		 (prefix[current_prefix.length()] == separator ||
 		  current_prefix == "")) {
 	    // We have, so start as many new branches as required.
-	    int current_prefix_length = current_prefix.Length();
+	    int current_prefix_length = current_prefix.length();
 	    current_prefix = prefix;
 	    if (current_prefix_length != 0) {
 		prefix = prefix.Mid(current_prefix_length + 1);
@@ -807,9 +809,9 @@ void MainFrm::FillTree()
 	// Otherwise, we must have moved up, and possibly then down again.
 	else {
 	    size_t count = 0;
-	    bool ascent_only = (prefix.Length() < current_prefix.Length() &&
+	    bool ascent_only = (prefix.length() < current_prefix.length() &&
 				current_prefix.StartsWith(prefix) &&
-				(current_prefix[prefix.Length()] == separator ||
+				(current_prefix[prefix.length()] == separator ||
 				 prefix == ""));
 	    if (!ascent_only) {
 		// Find out how much of the current prefix and the new prefix
@@ -820,7 +822,7 @@ void MainFrm::FillTree()
 		    if (prefix[i] == separator) count = i + 1;
 		}
 	    } else {
-		count = prefix.Length() + 1;
+		count = prefix.length() + 1;
 	    }
 
 	    // Extract the part of the current prefix after the bit (if any)
@@ -843,7 +845,7 @@ void MainFrm::FillTree()
 		wxString new_prefix = prefix.Mid(count);
 
 		// Add branches for this new part.
-		while (1) {
+		while (true) {
 		    // Extract the next bit of prefix.
 		    int next_dot = new_prefix.Find(separator);
 
@@ -876,9 +878,19 @@ void MainFrm::FillTree()
 	assert(bit != "");
 	wxTreeItemId id = m_Tree->AppendItem(current_id, bit);
 	m_Tree->SetItemData(id, new TreeData(label));
-	label->tree_id = id; // before calling SetTreeItemColour()...
-	SetTreeItemColour(label);
+	label->tree_id = id;
+	// Set the colour for an item in the survey tree.
+	if (label->IsEntrance()) {
+	    // Entrances are green (like entrance blobs).
+	    m_Tree->SetItemTextColour(id, wxColour(0, 255, 0));
+	} else if (label->IsSurface()) {
+	    // Surface stations are dark green.
+	    m_Tree->SetItemTextColour(id, wxColour(49, 158, 79));
+	}
     }
+
+    m_Tree->Expand(treeroot);
+    m_Tree->SetEnabled();
 }
 
 void MainFrm::SelectTreeItem(LabelInfo* label)
@@ -886,28 +898,14 @@ void MainFrm::SelectTreeItem(LabelInfo* label)
     m_Tree->SelectItem(label->tree_id);
 }
 
-void MainFrm::SetTreeItemColour(LabelInfo* label)
-{
-    // Set the colour for an item in the survey tree.
-
-    if (label->IsSurface()) {
-	m_Tree->SetItemTextColour(label->tree_id, wxColour(49, 158, 79));
-    }
-
-    if (label->IsEntrance()) {
-	// FIXME: making this red here doesn't match with entrance blobs
-	// being green...
-	m_Tree->SetItemTextColour(label->tree_id, wxColour(255, 0, 0));
-    }
-}
-
 void MainFrm::CentreDataset(Double xmin, Double ymin, Double zmin)
 {
     // Centre the dataset around the origin.
 
-    Double xoff = m_Offsets.x = xmin + (m_XExt / 2.0);
-    Double yoff = m_Offsets.y = ymin + (m_YExt / 2.0);
-    Double zoff = m_Offsets.z = zmin + (m_ZExt / 2.0);
+    Double xoff = xmin + (m_XExt / 2.0);
+    Double yoff = ymin + (m_YExt / 2.0);
+    Double zoff = zmin + (m_ZExt / 2.0);
+    m_Offsets.set(xoff, yoff, zoff);
 
     for (int band = 0; band < NUM_DEPTH_COLOURS + 1; band++) {
 	list<PointInfo*>::iterator pos = m_Points[band].begin();
@@ -1044,10 +1042,10 @@ void MainFrm::OpenFile(const wxString& file, wxString survey, bool delay)
 {
     wxBusyCursor hourglass;
     if (LoadData(file, survey)) {
-	if (wxIsAbsolutePath(file)) {
-	    m_history.AddFileToHistory(file);
+	if (wxIsAbsolutePath(m_File)) {
+	    m_history.AddFileToHistory(m_File);
 	} else {
-	    wxString abs = wxGetCwd() + wxString(FNM_SEP_LEV) + file;
+	    wxString abs = wxGetCwd() + wxString(FNM_SEP_LEV) + m_File;
 	    m_history.AddFileToHistory(abs);
 	}
 	wxConfigBase *b = wxConfigBase::Get();
@@ -1133,13 +1131,10 @@ void MainFrm::OnPrint(wxCommandEvent&)
 
 void MainFrm::OnPageSetup(wxCommandEvent&)
 {
-    m_pageSetupData.SetPrintData(m_printData);
-
-    wxPageSetupDialog pageSetupDialog(this, &m_pageSetupData);
-    pageSetupDialog.ShowModal();
-
-    m_printData = pageSetupDialog.GetPageSetupData().GetPrintData();
-    m_pageSetupData = pageSetupDialog.GetPageSetupData();
+    wxPageSetupDialog dlg(this, wxGetApp().GetPageSetupDialogData());
+    if (dlg.ShowModal() == wxID_OK) {
+	wxGetApp().SetPageSetupDialogData(dlg.GetPageSetupData());
+    }
 }
 
 void MainFrm::OnExport(wxCommandEvent&)
@@ -1170,7 +1165,7 @@ void MainFrm::OnClose(wxCloseEvent&)
 
 void MainFrm::OnAbout(wxCommandEvent&)
 {
-    AboutDlg dlg(this);
+    AboutDlg dlg(this, icon_path);
     dlg.Centre();
     dlg.ShowModal();
 }
@@ -1196,52 +1191,50 @@ void MainFrm::ClearCoords()
 
 void MainFrm::SetCoords(Double x, Double y)
 {
-    wxString str;
+    wxString s;
     if (m_Gfx->m_Metric) {
-	str.Printf(msg(/*  %d E, %d N*/338), int(x), int(y));
+	s.Printf(msg(/*  %d E, %d N*/338), int(x), int(y));
     } else {
-	str.Printf(msg(/*  %d E, %d N*/338),
-		   int(x / METRES_PER_FOOT), int(y / METRES_PER_FOOT));
+	s.Printf(msg(/*  %d E, %d N*/338),
+		 int(x / METRES_PER_FOOT), int(y / METRES_PER_FOOT));
     }
-    m_Coords->SetLabel(str);
+    m_Coords->SetLabel(s);
 }
 
 void MainFrm::SetAltitude(Double z)
 {
-    wxString str;
+    wxString s;
     if (m_Gfx->m_Metric) {
-	str.Printf("  %s %dm", msg(/*Altitude*/335),
-		   int(z));
+	s.Printf("  %s %dm", msg(/*Altitude*/335), int(z));
     } else {
-	str.Printf("  %s %dft", msg(/*Altitude*/335),
-		   int(z / METRES_PER_FOOT));
+	s.Printf("  %s %dft", msg(/*Altitude*/335), int(z / METRES_PER_FOOT));
     }
-    m_Coords->SetLabel(str);
+    m_Coords->SetLabel(s);
 }
 
 void MainFrm::ShowInfo(const LabelInfo *label)
 {
     assert(m_Gfx);
-	
+
     wxString str;
     if (m_Gfx->m_Metric) {
 	str.Printf(msg(/*  %d E, %d N*/338),
-		   int(label->x + m_Offsets.x),
-		   int(label->y + m_Offsets.y));
+		   int(label->x + m_Offsets.getX()),
+		   int(label->y + m_Offsets.getY()));
     } else {
 	str.Printf(msg(/*  %d E, %d N*/338),
-		   int((label->x + m_Offsets.x) / METRES_PER_FOOT),
-		   int((label->y + m_Offsets.y) / METRES_PER_FOOT));
+		   int((label->x + m_Offsets.getX()) / METRES_PER_FOOT),
+		   int((label->y + m_Offsets.getY()) / METRES_PER_FOOT));
     }
     m_StnCoords->SetLabel(str);
     m_StnName->SetLabel(label->text);
 
     if (m_Gfx->m_Metric) {
 	str.Printf("  %s %dm", msg(/*Altitude*/335),
-		   int(label->z + m_Offsets.z));
+		   int(label->z + m_Offsets.getZ()));
     } else {
 	str.Printf("  %s %dft", msg(/*Altitude*/335),
-		   int((label->z + m_Offsets.z) / METRES_PER_FOOT));
+		   int((label->z + m_Offsets.getZ()) / METRES_PER_FOOT));
     }
     m_StnAlt->SetLabel(str);
     m_Gfx->SetHere(label->x, label->y, label->z);
@@ -1268,7 +1261,7 @@ void MainFrm::ShowInfo(const LabelInfo *label)
 	    Double d_horiz = sqrt(dx*dx + dy*dy);
 	    Double dr = sqrt(dx*dx + dy*dy + dz*dz);
 
-	    Double brg = atan2(dx, dy) * 180.0 / M_PI;
+	    Double brg = deg(atan2(dx, dy));
 	    if (brg < 0) brg += 360;
 
 	    str.Printf(msg(/*From %s*/339), label2->text.c_str());
@@ -1309,21 +1302,19 @@ void MainFrm::ShowInfo(const LabelInfo *label)
 void MainFrm::DisplayTreeInfo(const wxTreeItemData* item)
 {
     const TreeData* data = static_cast<const TreeData*>(item);
-    if (data) {
-	if (data->IsStation()) {
-	    const LabelInfo * l = data->GetLabel();
-	    ShowInfo(l);
-	    m_Gfx->SetHere(l->x, l->y, l->z);
-	} else {
-	    m_StnName->SetLabel("");
-	    m_StnCoords->SetLabel("");
-	    m_StnAlt->SetLabel("");
-	    m_Gfx->SetHere();
-	    m_Dist1->SetLabel("");
-	    m_Dist2->SetLabel("");
-	    m_Dist3->SetLabel("");
-	    m_Gfx->SetHere();
-	}
+    if (data && data->IsStation()) {
+	const LabelInfo * l = data->GetLabel();
+	ShowInfo(l);
+	m_Gfx->SetHere(l->x, l->y, l->z);
+    } else {
+	m_StnName->SetLabel("");
+	m_StnCoords->SetLabel("");
+	m_StnAlt->SetLabel("");
+	m_Gfx->SetHere();
+	m_Dist1->SetLabel("");
+	m_Dist2->SetLabel("");
+	m_Dist3->SetLabel("");
+	m_Gfx->SetHere();
     }
 }
 
@@ -1356,6 +1347,7 @@ void MainFrm::OnFind(wxCommandEvent&)
 	re_flags |= wxRE_ICASE;
     }
 
+    bool substring = true;
     if (m_RegexpCheckBox->GetValue()) {
 	re_flags |= wxRE_EXTENDED;
     } else if (false /* simple glob-style */) {
@@ -1370,9 +1362,11 @@ void MainFrm::OnFind(wxCommandEvent&)
 	      break;
 	    case '*':
 	      pat += ".*";
+              substring = false;
 	      break;
 	    case '?':
 	      pat += '.';
+              substring = false;
 	      break;
 	    default:
 	      pat += ch;
@@ -1395,9 +1389,10 @@ void MainFrm::OnFind(wxCommandEvent&)
 	re_flags |= wxRE_BASIC;
     }
 
-    if (!true /* !substring */) {
+    if (!substring) {
 	// FIXME "0u" required to avoid compilation error with g++-3.0
 	if (pattern.empty() || pattern[0u] != '^') pattern = '^' + pattern;
+        // FIXME: this fails to cope with "\$" at the end of pattern...
 	if (pattern[pattern.size() - 1] != '$') pattern += '$';
     }
 
