@@ -4,7 +4,7 @@
 //  Main frame handling for Aven.
 //
 //  Copyright (C) 2000-2001, Mark R. Shinwell.
-//  Copyright (C) 2001-2003,2004 Olly Betts
+//  Copyright (C) 2001-2003,2004,2005 Olly Betts
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -28,37 +28,21 @@
 #include <wx/docview.h> // for m_FileHistory
 #include <wx/print.h>
 #include <wx/printdlg.h>
+
 #include "gfxcore.h"
 #include "message.h"
 #include "aventreectrl.h"
 #include "img.h"
 
 #include <list>
-#if 0 // if you turn this back on, reenable the check in configure.in too
-#ifdef HAVE_EXT_HASH_MAP
-#include <ext/hash_map>
-#elif defined HAVE_HASH_MAP
-#include <hash_map>
-#else
-#include <map>
-#define hash_map map
-#endif
-#endif
-
 using namespace std;
 
 #include <math.h>
 
-// This is for mingw32/Visual C++:
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-
 extern const int NUM_DEPTH_COLOURS;
 
 enum {
     menu_FILE_OPEN = 1000,
-    menu_FILE_OPEN_PRES,
     menu_FILE_PRINT,
     menu_FILE_PAGE_SETUP,
     menu_FILE_EXPORT,
@@ -106,14 +90,6 @@ enum {
     menu_VIEW_SIDE_PANEL,
     menu_VIEW_METRIC,
     menu_VIEW_DEGREES,
-    menu_PRES_CREATE,
-    menu_PRES_GO,
-    menu_PRES_GO_BACK,
-    menu_PRES_RESTART,
-    menu_PRES_RECORD,
-    menu_PRES_FINISH,
-    menu_PRES_ERASE,
-    menu_PRES_ERASE_ALL,
     menu_CTL_REVERSE,
     menu_CTL_CANCEL_DIST_LINE,
     menu_HELP_ABOUT,
@@ -195,7 +171,6 @@ private:
     wxSplitterWindow* m_Splitter;
     wxPanel* m_Panel;
     AvenTreeCtrl* m_Tree;
-    wxTreeItemId m_TreeRoot;
     wxTextCtrl* m_FindBox;
     wxStaticText* m_MousePtr;
     wxStaticText* m_Coords;
@@ -211,14 +186,13 @@ private:
     wxString m_Title, m_DateStamp;
     int separator; // character separating survey levels (often '.')
 
-    struct {
-	Double x, y, z;
-    } m_Offsets;
+    Vector3 m_Offsets;
 
-    wxPageSetupData m_pageSetupData;
+    wxPageSetupDialogData m_pageSetupData;
     wxPrintData m_printData;
 
-    void SetTreeItemColour(LabelInfo* label);
+    wxString icon_path;
+
     void FillTree();
     void ClearPointLists();
     bool LoadData(const wxString& file, wxString prefix = "");
@@ -262,6 +236,13 @@ public:
 
     void OnSetFocus(wxFocusEvent &) { if (m_Gfx) m_Gfx->SetFocus(); }
 
+    void OnKeyPress(wxKeyEvent &e) {
+	if (m_Gfx) {
+	    m_Gfx->SetFocus();
+	    m_Gfx->OnKeyPress(e);
+	}
+    }
+
     void OnPrintUpdate(wxUpdateUIEvent &ui) { ui.Enable(!m_File.empty()); }
     void OnExportUpdate(wxUpdateUIEvent &ui) { ui.Enable(!m_File.empty()); }
 
@@ -400,9 +381,9 @@ public:
     void SetCoords(Double x, Double y);
     void SetAltitude(Double z);
 
-    Double GetXOffset() const { return m_Offsets.x; }
-    Double GetYOffset() const { return m_Offsets.y; }
-    Double GetZOffset() const { return m_Offsets.z; }
+    Double GetXOffset() const { return m_Offsets.getX(); }
+    Double GetYOffset() const { return m_Offsets.getY(); }
+    Double GetZOffset() const { return m_Offsets.getZ(); }
 
     void SetMouseOverStation(LabelInfo* label);
 
@@ -445,7 +426,7 @@ public:
     void ShowInfo(const LabelInfo *label);
     void DisplayTreeInfo(const wxTreeItemData* data);
     void TreeItemSelected(wxTreeItemData* data);
-    wxPageSetupData * GetPageSetupData() { return &m_pageSetupData; }
+    wxPageSetupDialogData * GetPageSetupData() { return &m_pageSetupData; }
 
 private:
     DECLARE_EVENT_TABLE()
