@@ -27,9 +27,6 @@
 #include "aventypes.h"
 
 #include <math.h>
-#ifdef AVENPRES
-#include <stdio.h>
-#endif
 
 class Quaternion {
     Double w;
@@ -46,25 +43,13 @@ public:
 
     ~Quaternion() {}
 
-#ifdef AVENPRES
-    void Save(FILE* fp) const { //--Pres: FIXME: error handling
-	fwrite(&w, sizeof(Double), 1, fp);
-	v.Save(fp);
-    }
-
-    void Load(FILE* fp) { //--Pres: FIXME: error handling
-	fread(&w, sizeof(Double), 1, fp);
-	v.Load(fp);
-    }
-#endif
-
     Double magnitude() const {
 	Double mv = v.magnitude();
 #ifdef HAVE_HYPOT
 	return hypot(mv, w);
 #else
 	return sqrt(mv*mv + w*w);
-#endif	
+#endif
     }
 
     void normalise() {
