@@ -1,6 +1,6 @@
 /* readval.c
  * Routines to read a prefix or number from the current input file
- * Copyright (C) 1991-2001 Olly Betts
+ * Copyright (C) 1991-2002 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@
 # include <config.h>
 #endif
 
+#include <limits.h>
+#include <stddef.h> /* for offsetof */
+
 #include "cavern.h"
 #include "debug.h"
 #include "filename.h"
@@ -39,13 +42,9 @@
 
 int root_depr_count = 0;
 
-/* Dinky macro to handle any case forcing needed */
-#define docase(X) (pcs->Case == OFF ? (X) :\
-		   (pcs->Case == UPPER ? toupper(X) : tolower(X)))
-
-/* if prefix is omitted: if fOmit return NULL, otherwise use longjmp */
+/* if prefix is omitted: if f_optional return NULL, otherwise use longjmp */
 static prefix *
-read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
+read_prefix_(bool f_optional, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
 {
    prefix *back_ptr, *ptr;
    char *name;
@@ -59,19 +58,19 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
 #ifndef NO_DEPRECATED
    if (isRoot(ch)) {
       if (!fAllowRoot) {
-	 compile_error(/*ROOT is deprecated*/25);
-	 skipline();
+	 compile_error_skip(/*ROOT is deprecated*/25);
 	 LONGJMP(file.jbSkipLine);
       }
       if (root_depr_count < 5) {
 	 compile_warning(/*ROOT is deprecated*/25);
 	 if (++root_depr_count == 5)
-	    compile_warning(/*No further uses of this deprecated feature will be reported*/95);
+	    compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
       }
       nextch();
       ptr = root;
       if (!isNames(ch)) {
 	 if (!isSep(ch)) return ptr;
+	 /* Allow optional SEPARATOR after ROOT */
 	 nextch();
       }
       fImplicitPrefix = fFalse;
@@ -83,11 +82,14 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
 #endif
 
    i = 0;
+   name = NULL;
    do {
       fNew = fFalse;
       if (isSep(ch)) fImplicitPrefix = fFalse;
-      /* get a new name buffer for each level */
-      name = osmalloc(name_len);
+      if (name == NULL) {
+	 /* Need a new name buffer */
+	 name = osmalloc(name_len);
+      }
       /* i==0 iff this is the first pass */
       if (i) {
 	 i = 0;
@@ -96,7 +98,8 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
       while (isNames(ch)) {
 	 if (i < pcs->Truncate) {
 	    /* truncate name */
-	    name[i++] = docase(ch);
+	    name[i++] = (pcs->Case == LOWER ? tolower(ch) :
+			 (pcs->Case == OFF ? ch : toupper(ch)));
 	    if (i >= name_len) {
 	       name_len = name_len + name_len;
 	       name = osrealloc(name, name_len);
@@ -106,31 +109,40 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
       }
       if (i == 0) {
 	 osfree(name);
-	 if (!fOmit) {
+	 if (!f_optional) {
 	    if (isEol(ch)) {
 	       if (fSurvey) {
-		  compile_error(/*Expecting survey name*/89);
+		  compile_error_skip(/*Expecting survey name*/89);
 	       } else {
-		  compile_error(/*Expecting station name*/28);
+		  compile_error_skip(/*Expecting station name*/28);
 	       }
 	    } else {
-	       compile_error(/*Character `%c' not allowed in station name (use *SET NAMES to set allowed characters)*/7, ch);
+	       compile_error_skip(/*Character `%c' not allowed in station name (use *SET NAMES to set allowed characters)*/7, ch);
 	    }
-	    skipline();
 	    LONGJMP(file.jbSkipLine);
 	 }
 	 return (prefix *)NULL;
       }
 
       name[i++] = '\0';
-      name = osrealloc(name, i);
 
       back_ptr = ptr;
       ptr = ptr->down;
       if (ptr == NULL) {
 	 /* Special case first time around at each level */
+	 name = osrealloc(name, i);
+#ifdef CHASM3DX
+	 if (fUseNewFormat) {
+	    ptr = osnew(prefix);
+	 } else {
+	    /* only allocate the part of the structure we need... */
+	    ptr = (prefix *)osmalloc(offsetof(prefix, twig_link));
+	 }
+#else
 	 ptr = osnew(prefix);
+#endif
 	 ptr->ident = name;
+	 name = NULL;
 	 ptr->right = ptr->down = NULL;
 	 ptr->pos = NULL;
 	 ptr->shape = 0;
@@ -143,22 +155,41 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
 	    ptr->sflags |= BIT(SFLAGS_SUSPECTTYPO);
 	 back_ptr->down = ptr;
 	 fNew = fTrue;
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 	 if (fUseNewFormat) {
-	    create_twig(ptr,file.filename);
+	    create_twig(ptr, file.filename);
 	 }
 #endif
       } else {
+	 /* Use caching to speed up adding an increasing sequence to a
+	  * large survey */
+	 static prefix *cached_survey = NULL, *cached_station = NULL;
 	 prefix *ptrPrev = NULL;
 	 int cmp = 1; /* result of strcmp ( -ve for <, 0 for =, +ve for > ) */
+	 if (cached_survey == back_ptr) {
+	    cmp = strcmp(cached_station->ident, name);
+	    if (cmp <= 0) ptr = cached_station;
+	 }
 	 while (ptr && (cmp = strcmp(ptr->ident, name))<0) {
 	    ptrPrev = ptr;
 	    ptr = ptr->right;
 	 }
 	 if (cmp) {
 	    /* ie we got to one that was higher, or the end */
-	    prefix *newptr = osnew(prefix);
+	    prefix *newptr;
+	    name = osrealloc(name, i);
+#ifdef CHASM3DX
+	    if (fUseNewFormat) {
+	       newptr = osnew(prefix);
+	    } else {
+	       /* only allocate the part of the structure we need... */
+	       newptr = (prefix *)osmalloc(offsetof(prefix, twig_link));
+	    }
+#else
+	    newptr = osnew(prefix);
+#endif
 	    newptr->ident = name;
+	    name = NULL;
 	    if (ptrPrev == NULL)
 	       back_ptr->down = newptr;
 	    else
@@ -176,29 +207,37 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
 	       newptr->sflags |= BIT(SFLAGS_SUSPECTTYPO);
 	    ptr = newptr;
 	    fNew = fTrue;
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 	    if (fUseNewFormat) {
-	       create_twig(ptr,file.filename);
+	       create_twig(ptr, file.filename);
 	    }
 #endif
 	 }
+	 cached_survey = back_ptr;
+	 cached_station = ptr;
       }
       depth++;
-      fOmit = fFalse; /* disallow after first level */
+      f_optional = fFalse; /* disallow after first level */
    } while (isSep(ch));
+   if (name) osfree(name);
+
    /* don't warn about a station that is refered to twice */
    if (!fNew) ptr->sflags &= ~BIT(SFLAGS_SUSPECTTYPO);
 
    if (fNew) {
       /* fNew means SFLAGS_SURVEY is currently set */
-      ASSERT(TSTBIT(ptr->sflags, SFLAGS_SURVEY));
-      if (!fSurvey) ptr->sflags &= ~BIT(SFLAGS_SURVEY);
+      SVX_ASSERT(TSTBIT(ptr->sflags, SFLAGS_SURVEY));
+      if (!fSurvey) {
+	 ptr->sflags &= ~BIT(SFLAGS_SURVEY);
+	 if (TSTBIT(pcs->infer, INFER_EXPORTS)) ptr->min_export = USHRT_MAX;
+      }
    } else {
       /* check that the same name isn't being used for a survey and station */
       if (fSurvey ^ TSTBIT(ptr->sflags, SFLAGS_SURVEY)) {
 	 compile_error(/*`%s' can't be both a station and a survey*/27,
 		       sprint_prefix(ptr));
       }
+      if (!fSurvey && TSTBIT(pcs->infer, INFER_EXPORTS)) ptr->min_export = USHRT_MAX;
    }
 
    /* check the export level */
@@ -206,7 +245,7 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
    printf("R min %d max %d depth %d pfx %s\n",
 	  ptr->min_export, ptr->max_export, depth, sprint_prefix(ptr));
 #endif
-   if (ptr->min_export == 0) {
+   if (ptr->min_export == 0 || ptr->min_export == USHRT_MAX) {
       if (depth > ptr->max_export) ptr->max_export = depth;
    } else if (ptr->max_export < depth) {
       const char *filename_store = file.filename;
@@ -216,7 +255,7 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
       int level;
       for (level = ptr->max_export + 1; level; level--) {
 	 survey = survey->up;
-	 ASSERT(survey);
+	 SVX_ASSERT(survey);
       }
       s = osstrdup(sprint_prefix(survey));
       if (survey->filename) {
@@ -239,38 +278,37 @@ read_prefix_(bool fOmit, bool fSurvey, bool fSuspectTypo, bool fAllowRoot)
    return ptr;
 }
 
-/* if prefix is omitted: if fOmit return NULL, otherwise use longjmp */
+/* if prefix is omitted: if f_optional return NULL, otherwise use longjmp */
 extern prefix *
-read_prefix_survey(bool fOmit, bool fAllowRoot)
+read_prefix_survey(bool f_optional, bool fAllowRoot)
 {
-   return read_prefix_(fOmit, fTrue, fFalse, fAllowRoot);
+   return read_prefix_(f_optional, fTrue, fFalse, fAllowRoot);
 }
 
-/* if prefix is omitted: if fOmit return NULL, otherwise use longjmp */
+/* if prefix is omitted: if f_optional return NULL, otherwise use longjmp */
 extern prefix *
-read_prefix_stn(bool fOmit, bool fAllowRoot)
+read_prefix_stn(bool f_optional, bool fAllowRoot)
 {
-   return read_prefix_(fOmit, fFalse, fFalse, fAllowRoot);
+   return read_prefix_(f_optional, fFalse, fFalse, fAllowRoot);
 }
 
-/* if prefix is omitted: if fOmit return NULL, otherwise use longjmp */
+/* if prefix is omitted: if f_optional return NULL, otherwise use longjmp */
 /* Same as read_prefix_stn but implicit checks are made */
 extern prefix *
-read_prefix_stn_check_implicit(bool fOmit, bool fAllowRoot)
+read_prefix_stn_check_implicit(bool f_optional, bool fAllowRoot)
 {
-   return read_prefix_(fOmit, fFalse, fTrue, fAllowRoot);
+   return read_prefix_(f_optional, fFalse, fTrue, fAllowRoot);
 }
 
-/* if numeric expr is omitted: if fOmit return HUGE_REAL, else longjmp */
-extern real
-read_numeric(bool fOmit)
+/* if numeric expr is omitted: if f_optional return HUGE_REAL, else longjmp */
+static real
+read_number(bool f_optional)
 {
    bool fPositive, fDigits = fFalse;
    real n = (real)0.0;
    filepos fp;
    int ch_old;
 
-   skipblanks();
    get_pos(&fp);
    ch_old = ch;
    fPositive = !isMinus(ch);
@@ -297,7 +335,7 @@ read_numeric(bool fOmit)
    if (fDigits) return (fPositive ? n : -n);
 
    /* didn't read a valid number.  If it's optional, reset filepos & return */
-   if (fOmit) {
+   if (f_optional) {
       set_pos(&fp);
       return HUGE_REAL;
    }
@@ -311,11 +349,39 @@ read_numeric(bool fOmit)
    return 0.0; /* for brain-fried compilers */
 }
 
+extern real
+read_numeric(bool f_optional, int *p_n_readings)
+{
+   size_t n_readings = 0;
+   real tot = (real)0.0;
+
+   skipblanks();
+   if (!isOpen(ch)) {
+      real r = read_number(f_optional);
+      if (p_n_readings) *p_n_readings = (r == HUGE_REAL ? 0 : 1);
+      return r;
+   }
+   nextch();
+
+   skipblanks();
+   do {
+      tot += read_number(fFalse);
+      ++n_readings;
+      skipblanks();
+   } while (!isClose(ch));
+   nextch();
+
+   if (p_n_readings) *p_n_readings = n_readings;
+   /* FIXME: special averaging for bearings ... */
+   /* And for percentage gradient */
+   return tot / n_readings;
+}
+
 /* read numeric expr or omit (return HUGE_REAL); else longjmp */
 extern real
-read_numeric_or_omit(void)
+read_numeric_or_omit(int *p_n_readings)
 {
-   real v = read_numeric(fTrue);
+   real v = read_numeric(fTrue, p_n_readings);
    if (v == HUGE_REAL) {
       if (!isOmit(ch)) {
 	 compile_error_token(/*Expecting numeric field, found `%s'*/9);
@@ -358,11 +424,11 @@ read_string(char **pstr, int *plen)
 
    skipblanks();
    if (ch == '\"') {
+      /* String quoted in "" */
       nextch();
       while (1) {
 	 if (isEol(ch)) {
-	    compile_error(/*Missing &quot;*/69);
-	    skipline();
+	    compile_error_skip(/*Missing &quot;*/69);
 	    return;
 	 }
 
@@ -372,11 +438,11 @@ read_string(char **pstr, int *plen)
 	 nextch();
       }
    } else {
+      /* Unquoted string */
       while (1) {
 	 if (isEol(ch) || isComm(ch)) {
 	    if (!*pstr || !(*pstr)[0]) {
-	       compile_error(/*Expecting string field*/121);
-	       skipline();
+	       compile_error_skip(/*Expecting string field*/121);
 	    }
 	    return;
 	 }
