@@ -622,6 +622,7 @@ draw_label(Window window, GC gc, int x, int y,
 
       if (x >= oldwidth || y >= oldheight || y + fontheight <= 0) return;
 
+      /* FIXME: should cache strwidth for each label for efficiency */
       strwidth = XTextWidth(fontinfo, string, length);
 
       if (x + strwidth <= 0) return;
@@ -1186,8 +1187,11 @@ main(int argc, char **argv)
    /* create a window with a size such that it fits on the screen */
    myhint.x = 0;
    myhint.y = 0;
-   myhint.width = WidthOfScreen(DefaultScreenOfDisplay(mydisplay))-5;
-   myhint.height = HeightOfScreen(DefaultScreenOfDisplay(mydisplay))-25;
+   myhint.width = WidthOfScreen(DefaultScreenOfDisplay(mydisplay));
+   myhint.height = HeightOfScreen(DefaultScreenOfDisplay(mydisplay));
+   if (myhint.width < 640 || myhint.height < 480) pda = 1;
+   myhint.width -= 5;
+   myhint.height -= 25;
    myhint.flags = /* PPosition | */ PSize;
 
    if (have_double_buffering) {
